/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "pedometer.h"
#include "hal_data.h"
#include "application_dispatcher_thread.h"
#include "common.h"

extern activity_data g_activity_data;
static activity_info_t s_activity_info;

#define RUN_AVG_SCALE_THR  1000000 // 2^11 *2^11 * 1000 int32 capacity, so if input > thr scaled down 1st, else scaled after
#define PEAK_MET_KAVG_TRHESHOLD  50 // for thr comparison
#define PEAK_MET_KAVG_UP  7    // Low level which signal is required
#define PEAK_MET_KAVG_DOWN  10 // to go below prior to next compare to Thr
//#define DIFF_THR  52429  // In LV set to 0.2 with MaxMag of 48 (3*4^2). The acc range = +/- 2048 so MaxMag = 3*2048^2 and Thr = 3*2048^2 * 0.2/48
#define DIFF_THR  32768

// Updated to 5 seconds
#define CADENCE_MINIMAL (188)   // ~ 3 seconds based on 16ms ticker

// use LED2 in the board to show pedometer_update been called
#define PEDOMETER_LED_FEEDBACK

enum PEAK_METHOD_STATE {
    TM_BELOW_THR,
    TM_VERIFY_PEAK,
    TM_GO_BELOW
};

// forward declarations
static int32_t running_avg(int32_t kavg, int32_t prev_out, int32_t input, bool first_call);
static int32_t running_avg_biased(int32_t kavg_up, int32_t kavg_down, int32_t prev_out, int32_t input, bool first_call);
static uint8_t calculate_cadence (step_info_t *step);
static void pedometer_update (activity_info_t *activity);
static int32_t pedometer_tracking ( activity_info_t *act, int32_t biased_avg, int32_t Threshold_rAvg );
static uint8_t estimate_LAW (int32_t Last_Pk);

void pedometer (xyz_count_t *data, int count)
{
    static bool main_filter_first_call = true;
    static bool biased_first_call = true;
    static bool threshold_first_call = true;
    static int32_t biased_avg, Threshold_rAvg;

    // TODO: this makes for some nice Qf states, signals, and trans
    // assume that all the axis data will be sent
    for (int i = 0; i < count; i++)
    {
        // extract X,Y,Z data from accel.
        s_activity_info.count.x = data[i].x;
        s_activity_info.count.y = data[i].y;
        s_activity_info.count.z = data[i].z;

        // get raw accelerometer data and sum the squares for magnitude
        s_activity_info.squared = (uint32_t)(s_activity_info.count.x * s_activity_info.count.x);
        s_activity_info.level = (int32_t)s_activity_info.squared;
        s_activity_info.squared = (uint32_t)(s_activity_info.count.y * s_activity_info.count.y);
        s_activity_info.level += (int32_t)s_activity_info.squared;
        s_activity_info.squared = (uint32_t)(s_activity_info.count.z * s_activity_info.count.z);
        s_activity_info.level += (int32_t)s_activity_info.squared;

        // Running avg used as filter to acc inputs. (present for Acc data capture)
        s_activity_info.level_avg = running_avg(100, s_activity_info.level_avg, s_activity_info.level, main_filter_first_call);
        main_filter_first_call = false;

        // Running avg used as threshold comparator to signal
        Threshold_rAvg = running_avg(PEAK_MET_KAVG_TRHESHOLD, Threshold_rAvg, s_activity_info.level_avg, threshold_first_call);
        threshold_first_call = false;

        // Biased avg used
        biased_avg = running_avg_biased(PEAK_MET_KAVG_UP, PEAK_MET_KAVG_DOWN, biased_avg, s_activity_info.level_avg, biased_first_call);
        biased_first_call = false;

        pedometer_tracking ( &s_activity_info, biased_avg, Threshold_rAvg );

        // increment tick_counter which represents Tsam
        s_activity_info.step.tick++;

    }   // end for loop
}

static volatile uint32_t s_steps = 0;
static volatile int32_t s_threshold = 0;

static int32_t pedometer_tracking ( activity_info_t *act, int32_t biased_avg, int32_t Threshold_rAvg )
{
    static enum PEAK_METHOD_STATE  Peak_Method_State = TM_BELOW_THR;
    static int16_t WinCnt = 0;
    static int32_t Last_Pk = 0;
    static bool update = false;
    uint8_t Est_LAW = 0;
    //static volatile uint16_t s_steps = 0;

    // for debugging
    s_steps = act->step.total;

    switch (Peak_Method_State)
    {
        case TM_BELOW_THR:
            s_threshold = act->level_avg - Threshold_rAvg;
            if ( (act->level_avg - Threshold_rAvg) > DIFF_THR )
            {
                WinCnt = 0;
                Peak_Method_State = TM_VERIFY_PEAK;
            }
            else
            {
                // detect stopped
                if (!act->step.stopped &&
                        // at least one step every 5 sec or we will consider subject stopped
                        ((act->step.tick  - act->step.tick_last) > CADENCE_MINIMAL))
                {
                    // effectively stopped subject
                    act->step.cadence = 0;
                    act->step.stopped = true;
                    // let anybody know
                    pedometer_update (act);
                }
            }
            break;

        case TM_VERIFY_PEAK:
            if ( act->level_avg > Last_Pk )
            {
                WinCnt = 0;
                Last_Pk = act->level_avg;
            }
            else
            {
                WinCnt++;
                Est_LAW = estimate_LAW (Last_Pk);
                if ( WinCnt >= Est_LAW)
                {
                  Peak_Method_State = TM_GO_BELOW;
                  WinCnt = 0;

                  // we are moving again ....
                  act->step.stopped = false;

                  // window of time when we expect this to happen
                  if ((((act->step.tick - act->step.tick_last) > STEP_MIN_TICK_COUNT) &&
                          ((act->step.tick - act->step.tick_last) < STEP_MAX_TICK_COUNT)) || !update)
                  {
                      act->step.possible_steps++;
                      if (act->step.possible_steps > STEP_MIN_FOR_ACTIVITY)
                      {
                          update = true;
                      }
                      if (update)
                      {   // These are now the same
                          act->step.count++;
                          act->step.total = act->step.count;
                          // speed at which this is taking place in repetitions per minute
                          act->step.cadence = calculate_cadence (&act->step);
                          // perform an update
                          pedometer_update (act);
                      }
                      act->step.tick_last = act->step.tick;
                  }

                }
            }
            break;

        case TM_GO_BELOW:
            WinCnt = 0;
            if ( act->level_avg < biased_avg  )
            {
                Peak_Method_State = TM_BELOW_THR;
            }
            else
            {        // at least one step every 5 sec or we will consider subject stopped
                if ((act->step.tick  - act->step.tick_last) > CADENCE_MINIMAL)
                {
                    // effectively stopped subject
                    act->step.cadence = 0;
                    act->step.stopped = true;
                    act->step.possible_steps = 0;
                    update = false;
                    // let anybody know
                    pedometer_update (act);
                    // return to default state
                    Peak_Method_State = TM_BELOW_THR;
                }
            }
            break;
    }   // end state machine switch

    return 0;
}

static void pedometer_update_feedback (void)
{
#ifdef PEDOMETER_LED_FEEDBACK
    const ioport_instance_t *p = &g_ioport;
    ioport_level_t level;
    // just show what we are doing
    p->p_api->pinRead(IOPORT_PORT_03_PIN_05, &level);
    level = IOPORT_LEVEL_LOW == level ? IOPORT_LEVEL_HIGH : IOPORT_LEVEL_LOW;
    p->p_api->pinWrite(IOPORT_PORT_03_PIN_05, level);
#endif
}

static void pedometer_update_ble (activity_info_t *activity)
{
    // update values
    tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
    g_activity_data.calories = (uint16_t) activity->calories;
    g_activity_data.distance = (uint16_t) (activity->distance * 100.0f);
    g_activity_data.steps = (uint16_t)activity->step.total;
    tx_mutex_put(&g_data_mutex);
}

static void pedometer_update (activity_info_t *activity)
{
    // update calories and distance every N steps
    if (0 == (activity->step.total % STEPS_UNTIL_UPDATE))
    {
        // x
        // activity->.step.total - populated in main pedometer routine
        // y
        activity->distance = (float)activity->step.total / STEPS_PER_MILE;
        // z
        activity->calories = activity->distance * CALORIES_BURNED_PER_MILE;
        // do some feedback on the board
        pedometer_update_feedback ();
        // push the data through BLE
        pedometer_update_ble (activity);
    }
}

static uint8_t calculate_cadence (step_info_t *step)
{
    /** instantaneousCadenceValue:
     * The instantaneous cadence,
     * Unit is in 1/minute (or RPM) with a resolutions of 1 (1/min or 1 RPM)
     */
    uint32_t freq = ACC_RATE;
    uint32_t delta = step->tick - step->tick_last;
    uint8_t cadence = (uint8_t)((freq * 60U) / delta);

    return cadence;
}

static int32_t running_avg(int32_t kavg, int32_t prev_out, int32_t input, bool first_call)
{
    int32_t avg_result = 0;

    if (first_call)
    {
        // first time calling function
        avg_result = input;
    }
    else
    {
        // avoid overflowing int32
        input /= 1000;
        prev_out /= 1000;
        prev_out = kavg * input + (1000 - kavg) * prev_out;
        avg_result = prev_out;
    }

    // return averaged result
    return avg_result;
}

static int32_t running_avg_biased(int32_t kavg_up, int32_t kavg_down, int32_t prev_out, int32_t input, bool first_call)
{
    int32_t avg_result = 0;
    int32_t Loc_kavg = 0;

    if (first_call)
    {
        // first time calling function
        avg_result = input;
    }
    else
    {
        if ( input > prev_out )
        {
            Loc_kavg = kavg_up;
        }
        else
        {
            Loc_kavg = kavg_down;
        }
        if ( (input > RUN_AVG_SCALE_THR) || (prev_out > RUN_AVG_SCALE_THR) )
        {
            input /= 1000;
            prev_out /= 1000;
            prev_out = Loc_kavg * input + (1000 - Loc_kavg) * prev_out;
        }
        else
        {
            prev_out = Loc_kavg * input + (1000 - Loc_kavg) * prev_out;
            prev_out /= 1000;
        }
        avg_result = prev_out;
    }

    // return averaged result
    return avg_result;
}

static uint8_t estimate_LAW (int32_t Last_Pk)
{
    (void) ((Last_Pk));
    return 25;
}

