package com.xaplos.hcsolution;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;
import java.util.concurrent.locks.ReentrantLock;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.util.Log;

abstract class BLEDevice {
	private static final String TAG = "BLEDevice";
	private static final UUID uuidClientCharacteristicConfiguration = UUID.fromString("00002902-0000-1000-8000-00805f9b34fb");

	private BluetoothGatt mGatt;
	private final HashMap<UUID, BLECharacteristic> mCharacteristicHash = new HashMap<>();

    private Timer mNotificationWatchdog = null;
    private Date mNotificationStart = null;
    private Date mNotificationRecent = null;

	public abstract String getDeviceName();
	public abstract UUID getServiceUuid();
    public abstract List<BLECharacteristic>getCharacteristics();

    // IntelliJ thinks this is unused, but it is used
    @SuppressWarnings("unused")
    BLEDevice() {
        Log.e(TAG, "is this called?");

        for(BLECharacteristic c : getCharacteristics()) {
            mCharacteristicHash.put(c.getCharacteristicUuid(), c);
        }
    }

    public BLECharacteristic getCharacteristic(UUID uuid) {
        return mCharacteristicHash.get(uuid);
    }

    private class WatchDogTask extends TimerTask {
        @Override
        public void run() {
            Date now = new Date();
            long startDelta = (now.getTime() - mNotificationStart.getTime()) / 1000;

            if (mNotificationRecent == null) {
                Log.e(TAG, "notifications never started, waited " + startDelta + "s");
            }
            else {
                long timeDelta = (now.getTime() - mNotificationRecent.getTime()) / 1000;
                Log.e(TAG, "notifications last seen " + timeDelta + "s ago, started " + startDelta + "s ago");
            }
        }
    }

    private final ReentrantLock mWatchdogLock = new ReentrantLock();

    public void petWatchdog () {
        mWatchdogLock.lock();

        try {
            mNotificationRecent = new Date();
            runWatchdog();
        }
        finally {
            mWatchdogLock.unlock();
        }
    }

    private void runWatchdog() {
        mWatchdogLock.lock();

        try {
            if (mNotificationWatchdog == null) {
                mNotificationWatchdog = new Timer();
            }

            if (mNotificationStart == null) {
                mNotificationStart = new Date();
                Log.d(TAG, "starting watchdog at " + mNotificationStart);
            }

            mNotificationWatchdog.cancel();
            mNotificationWatchdog = new Timer();
            mNotificationWatchdog.schedule(new WatchDogTask(), 30 * 1000);
        }
        finally {
            mWatchdogLock.unlock();
        }
    }

    public void resetWatchdog() {
        mWatchdogLock.lock();

        try {
            Log.d(TAG, "resetting watchdog");
            if (mNotificationWatchdog != null) {
                mNotificationWatchdog.cancel();
            }

            mNotificationStart = null;
        }
        finally {
            mWatchdogLock.unlock();
        }
    }

	public void setCharacteristicNotification(BluetoothGattCharacteristic characteristic, @SuppressWarnings("SameParameterValue") boolean enable) {
		Log.d(TAG, (enable ? "enable " : "disable ") + "notification for " + characteristic.getUuid().toString());
        runWatchdog();

		if (mGatt == null) {
			Log.e(TAG, "setCharacteristicNotification: null gatt");
			return;
		}
		
		if (!mGatt.setCharacteristicNotification(characteristic, enable)) {
			Log.w(TAG, "setCharacteristicNotification failed");
		}
		else {
            setNotificationsEnable(characteristic, enable);
        }
	}

	private void setNotificationsEnable(BluetoothGattCharacteristic characteristic, boolean enable) {
		BluetoothGattDescriptor descriptor = characteristic.getDescriptor(uuidClientCharacteristicConfiguration);

        if (descriptor == null) {
            Log.e(TAG, "setNotificationsEnable: ClientCharacteristicConfiguration is null for " + characteristic.getUuid().toString());
        }
        else {
            if (enable) {
                descriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
            } else {
                descriptor.setValue(BluetoothGattDescriptor.DISABLE_NOTIFICATION_VALUE);
            }

            mGatt.writeDescriptor(descriptor);
        }
	}

	public void writeBytesToDescriptor(UUID writableUuid, byte[] value) {
        if (mGatt == null) {
            Log.e(TAG, "mGatt is null, cannot write to descriptor");
            return;
        }

        UUID serviceUuid = getServiceUuid();
        BluetoothGattService service = mGatt.getService(serviceUuid);

        if (service == null) {
            Log.e(TAG, "unable to get service " + serviceUuid.toString());
            return;
        }

        BluetoothGattCharacteristic characteristic = service.getCharacteristic(writableUuid);

        if (characteristic == null) {
            Log.e(TAG, "unable to get characteristic " + writableUuid.toString());
            return;
        }

//		characteristic.setWriteType(BluetoothGattCharacteristic.WRITE_TYPE_NO_RESPONSE);
        characteristic.setValue(value);
        mGatt.writeCharacteristic(characteristic);
	}

	public BluetoothGatt getGatt() {
		return mGatt;
	}
	
	public void setGatt(BluetoothGatt gatt) {
		mGatt = gatt;
	}
}
