package com.xaplos.hcsolution;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;


public class BLEManager {
    private static BLEManager ourInstance = new BLEManager();
    private final String TAG = "BLEManager";
    private Messenger mBLEService;
    private final Messenger mMessenger = new Messenger(new BLEServiceHandler(this));
    private Activity mActivity;
    private BLEManagerListener mListener;
    private ArrayList<BLEManagerNotifyListener> mNotifyListeners = new ArrayList<>();
    private final static int REQUEST_ENABLE_BLUETOOTH = 1;
    private boolean mBound = false;
    private HashMap<String, Bundle> mDevices = new HashMap<>();

    //
    // A listener for state related changes
    //
    public interface BLEManagerListener {
        void onDiagnosticMessage(String message);
        void onDeviceDiscovered(Bundle device);
        void onDeviceConnected();
        void onDeviceDisconnected();
    }

    //
    // Any number of listeners for streaming data
    //
    public interface BLEManagerNotifyListener {
        void onNotification(Bundle notification);
    }

    public static BLEManager getInstance() {
        return ourInstance;
    }

    private BLEManager() {
    }

    public void setActivity(Activity activity) {
        Log.d(TAG, "setActivity(" + activity + ")");
        mActivity = activity;
    }

    public void setListener(BLEManagerListener listener) {
        mListener = listener;
    }

    public void addNotifyListener(BLEManagerNotifyListener listener) {
        mNotifyListeners.add(listener);
    }

    public void start () {
        startBleService();
    }

    private void setMessage(String text) {
        if (mListener != null) {
            mListener.onDiagnosticMessage(text);
        }
    }

    private void processNotification(Message msg) {
        Bundle bundle = msg.getData();

        for (BLEManagerNotifyListener listener: mNotifyListeners) {
            listener.onNotification(bundle);
        }
    }

    private void processDiagnostic(Message msg) {
        BLEService.Diagnostic diagnostic = BLEService.Diagnostic.values()[msg.arg1];
        setMessage(diagnostic.toString());
        Bundle bundle = msg.getData();

        switch (diagnostic) {
            case SCANNING:
                mDevices.clear();
                break;

            case DEVICE_FOUND:
                String address = bundle.getString("address");

                if (!mDevices.containsKey(address)) {
                    mDevices.put(address, bundle);

                    if (mListener != null) {
                        mListener.onDeviceDiscovered(bundle);
                    }
                }

                break;

            case DEVICE_NOT_FOUND:
                Log.e(TAG, "todo: DEVICE_NOT_FOUND");
                break;

            case DEVICE_CONNECTED:
                if (mListener != null) {
                    mListener.onDeviceConnected();
                }
                break;

            case DEVICE_DISCONNECTED:
                resetScan();

                if (mListener != null) {
                    mListener.onDeviceDisconnected();
                }
                break;

            case DEVICE_NOT_SUPPORTED:
                resetScan();
                break;

            case BLE_NOT_SUPPORTED:
                break;

            case BLE_NOT_ENABLED:
                break;
        }
    }

    private void resetScan() {
        mDevices.clear();
    }

    public void startScan() {
        sendBLERequest(BLEService.Request.START_SCANNING);
    }

    public void disconnect() {
        sendBLERequest(BLEService.Request.DISCONNECT);
    }

    public void connect(Bundle device) {
        sendBLERequest(BLEService.Request.CONNECT, device);
    }

    private void sendBLERequest (BLEService.Request action) {
        sendBLERequest(action, null);
    }

    /*
     * Send a request to the BLEService
     */
    private void sendBLERequest (BLEService.Request action, Bundle payload) {
        Log.d(TAG, "send " + action.name());
        Message msg = Message.obtain(null, action.ordinal(), 0, 0);
        msg.replyTo = mMessenger;

        if (payload != null) {
            msg.setData(payload);
        }

        try {
            mBLEService.send(msg);
        }
        catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /*
     * Ask BLEService to write data to descriptorUuid
     */
    private void sendWriteDescriptorRequest(UUID descriptorUuid, byte[] data) {
        Bundle bundle = new Bundle();
        bundle.putString("uuid", descriptorUuid.toString());
        bundle.putByteArray("bytes", data);
        sendBLERequest(BLEService.Request.WRITE_DESCRIPTOR, bundle);
    }

    /**
     * Handle messages from BLEService
     */
    static class BLEServiceHandler extends Handler {
        private static final String TAG = "BLEServiceHandler";
        private final WeakReference<BLEManager> mManager;

        BLEServiceHandler(BLEManager manager) {
            mManager = new WeakReference<>(manager);
        }

        @Override
        public void handleMessage(Message msg) {
            BLEService.Response response = BLEService.Response.values()[msg.what];
            BLEManager manager = mManager.get();

            if (manager == null) {
                Log.e(TAG, "BLEManager should not be null");
                return;
            }

            switch(response) {
                case NEED_BLUETOOTH:
                    Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                    Activity activity = manager.mActivity;

                    if (activity == null) {
                        Log.e(TAG, "Activity should not be null");
                        return;
                    }

                    activity.startActivityForResult(enableBtIntent, REQUEST_ENABLE_BLUETOOTH);
                    break;

                case NOTIFICATION:
                    manager.processNotification(msg);
                    break;

                case DIAGNOSTIC:
                    manager.processDiagnostic(msg);
                    break;

                default:
                    Log.e(TAG, "unknown response: " + response.ordinal() + " " + response.toString());
                    break;
            }
        }
    }

    private void startBleService() {
        Log.d(TAG, "calling bindService");
        mActivity.startService(new Intent(mActivity, BLEService.class));
        mActivity.bindService(new Intent(mActivity, BLEService.class), mConnection, Context.BIND_AUTO_CREATE);
    }

    private void stopBleService() {
        if (mBound) {
            Log.d(TAG, "calling unbindService");
            mActivity.unbindService(mConnection);
            mBound = false;
        }
    }

    private final ServiceConnection mConnection = new ServiceConnection() {
        public void onServiceConnected(ComponentName className, IBinder service) {
            mBLEService = new Messenger(service);
            mBound = true;
            Log.d(TAG, "onServiceConnected");
        }

        public void onServiceDisconnected(ComponentName className) {
            mBLEService = null;
            mBound = false;
            Log.d(TAG, "onServiceDisconnected");
        }
    };
}
