/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * adc_process.h
 *
 *  Created on: Jul 28, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */

#include "hal/adc_ssp.h"
#include "config.h"
#ifndef ADC_H_
#define ADC_H_

#ifdef USE_DUMMY_DATA
int i;                                                          //Used for dummy data
#endif
/*
 * adc_state_t is the states that the adc can be in
 */
typedef enum
{
    ADC_BATT_SAMPLE,                                            //Sample only the battery adc value
    ADC_BG_BATT_SAMPLE,                                         //Sample the battery and blood glucose adc values
    ADC_MAX_STEPS,                                              //Max number of states
}adc_state_t;

/*
 * bgm_state_t is the states that the blood glucose meter can be in
 */
typedef enum
{
    BGM_IDLE_STATE,
    BGM_WAITING_FOR_STRIP_STATE,
    BGM_STRIP_INSERTED_STATE,
    BGM_MEASURE_STATE,
    BGM_FINISH_STATE,
    BGM_FAILED_STATE,
    BGM_MAX_STATES                                              // Max number of states
} bgm_state_t;

/*
 * adc_val_t enumerates the number of channels from the adc
 */
typedef enum
{
    ADC_BG_VALUE,                                               //The blood glucose value
    ADC_BATT_VALUE,                                             //The battery value
    ADC_VALUE_COUNT                                             //Count of adc values
}adc_val_t;

/*
 * results_t contains variables for holding the results of the adc sampling and conversion functions
 */
typedef struct
{
    int bg_adc_val;                                             //Blood glucose adc value
    int blood_glucose_value;                                          //Converted read out blood glucose value
    int batt_value;                                             //Battery adc value
} results_t;

/*
 * adc_sample_context_t contains all necessary variables for adc.c functions
 */
typedef struct
{
    const pmcu_resource_t *res;                                 //Mcu specific resource context
    TX_MUTEX *mutex;
    adc_state_t state;                                          //Contains the current adc_state_t
    adc_value_t val[ADC_VALUE_COUNT];                           //Contains adc value structs from adc_ssp.h equal to the number of used channels
}adc_sample_context_t;

/*
 * MCU specific resources
 */
extern const pmcu_resource_t adc_mcu_res;

/*
 * ADC resource control context
 */
extern adc_sample_context_t  adc_sample_context;


/**
 * @brief adc_notify_init initializes all related to PULSE ADC data notification
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int adc_notify_init (const adc_sample_context_t *c);

/**
 * @brief adc_notify_close closes all related to ADC data notification
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int adc_notify_close (const adc_sample_context_t *c);

/**
 * @brief adc_notify notifies of a particular ADC value been sensed
 *
 * @param c pointer to the MCU resource control context
 * @param event contains the flag corresponding to the ADC data that is been acquired
 *
 * @return 0 if no error, error code otherwise
 */
int adc_notify (const adc_sample_context_t *c, int event);

/**
 * @brief adc_init initializes all related to ADC sensing
 *
 * @param none
 *
 * @return 0 if no error, error code otherwise
 */
int adc_init (adc_sample_context_t *c);

/**
 * @brief adc_close closes all related to ADC sensing
 *
 * @param none
 *
 * @return 0 if no error, error code otherwise
 */
int adc_close (const adc_sample_context_t *c);

/**
 * @brief adc_open opens ADC peripheral, leaves it ready
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int adc_open (const adc_sample_context_t *c);

/**
 * @brief adc_clos opens ADC peripheral, leaves it ready
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int adc_clos (const adc_sample_context_t *c);

/**
 * @brief sample_timer_open opens 2ms timer and starts it
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int sample_timer_open (const adc_sample_context_t *c);

/**
 * @brief sample_timer_close closes 2ms timer and starts it
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int sample_timer_close (const adc_sample_context_t *c);

/**
 * @brief adc_convert_start fires up the ADC conversion
 *
 * @param c pointer to the MCU resource control context
 *
 * @return none
 */
int adc_scan_start (const adc_sample_context_t *c);

/**
 * @brief adc_wait_for_event waits for ever for a ADC value to be collected event
 *
 * @param event pointer to the event storage
 * @param bg_val pointer to the blood glucose ADC value
 * @param batt_val pointer to the battery ADC value
 *
 * @return 0 if no error, error code otherwise
 */
int adc_wait_for_event(adc_sample_context_t *c, ULONG *event, int *bg_val, int *batt_val);

/**
 * @brief adc_read reads the converted value from the ADC line
 *
 * @param c pointer to the adc sample context
 * @param c pointer to the adc sample return storage area
 *
 * @return 0 if no error, error code otherwise
 */
int adc_read (adc_sample_context_t *c, adc_value_t *val);

/**
 * @brief adc_read_all reads all values from the ADC line
 *
 * @param c pointer to the adc sample context
 *
 * @return 0 if no error, error code otherwise
 */
int adc_read_all(adc_sample_context_t *c);

/**
 * @brief adc_sample_handle samples the ADC channel
 *
 * @param c pointer to the adc context
 *
 * @return none
 */
int adc_sample_handle (adc_sample_context_t *c);

/**
 * @brief adc_percent returns the percend that the ADC is
 *
 * @param adc ADC raw count value
 *
 * @return percent 0 - 100
 */
float adc_percent (int adc);

#endif /* ADC_H_ */
