/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * adc_process.c
 *
 *  Created on: Jul 29, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */

#include "adc_process.h"
#include "application_dispatcher_thread.h"
#include "common.h"

extern healthcare_data g_healthcare_data;
adc_peak_t adc_peak;

void adc_update_bgm_state(bgm_state_t state);
void adc_update_bgm_state(bgm_state_t state)
{
    // update values
    tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
    g_healthcare_data.bgm_state = (uint16_t) state;
    tx_mutex_put(&g_data_mutex);
}

/**
 * @brief adc_process_bg process blood glucose adc values coming from adc_thread_entry
 * @param value is the blood glucose adc value
 * @param bg is a pointer to the location the valid converted blood glucose value is to be saved
 */
int adc_process_bg(int value, int *bg)
{

    int err;
    adc_peak_t *adc_p = &adc_peak;
    adc_sample_context_t *c = &adc_sample_context;

    *bg = 0;                                                                //Init bg pointer value

    err = adc_track_peak (value, adc_p);                                          //Detect adc values' peak and track how long its been since the peak

    if (adc_p->peak_time_elapsed >= TWO_SECONDS)                            //If its been two seconds since the peak...
    {
        c->state = ADC_BATT_SAMPLE;                                         //Reset the adc step to just sample the battery value after this process
        *bg = calculate_blood_glucose (value);                              //Convert the blood glucose adc value to the standard read out value

#ifdef USE_DUMMY_DATA
        test_timer.p_api->stop (test_timer.p_ctrl);                         //If using dummy data, stop the dummy timer
#endif
    }
    return err;
}

/**
 * @brief calculate_blood_glucose converts the blood glucose adc value to the standard read out value
 * @param value is the blood glucose adc value
 * @return the converted value
 */
int calculate_blood_glucose(int value)
{
    float voltage;
    float bgm;

    adc_update_bgm_state(BGM_FINISH_STATE);
    // 1. convert ADC value to voltage
    voltage = 3.3f - (((16383.0f - (float)value) / 16383.0f)) * 3.3f;
    // 2. BGM transfer funciton from basic lookup tables
    //    based on captured sample data
    //bgm = (418.09f * voltage) - 692.47f;
    bgm = (591.379f * voltage) - 998.759f;
    // convert float to int; don't worry about rounding
    return (int) bgm;                                                     //Place holder for actual algorithm
}

/**
 * @brief adc_track_peak compares the current adc value to past values to find the peak, and measures the elapsed time since the peak
 * @param value is the current adc value
 * @param adc_p is the struct in which the peak value and elapsed time is saved
 */
int adc_track_peak(int value, adc_peak_t *adc_p)
{
    int err;
    adc_sample_context_t *c = &adc_sample_context;

    if (adc_p->peak > DETECT_THRESHOLD || value > DETECT_THRESHOLD)         //Only run if blood has been inserted into the strip (adc values breaks DETECT_THRESHOLD)
    {
        if (adc_p->peak == 0)                                               //After the first sample...
        {
            err = tx_event_flags_set (c->res->event, DISPLAY_BG_EVENT, TX_OR);    //Tell display_thread_entry to display --- and wait for the blood glucose value via queue
            adc_update_bgm_state(BGM_MEASURE_STATE);
        }
        if (value > adc_p->peak)                                            //If the current adc value is greater than the saved value...
        {
            adc_p->peak = value;                                            //Update the peak value
            adc_p->peak_time_elapsed = 0;                                   //Reset the elapsed time since peak
        }
        else                                                                //else...
        {
            adc_p->peak_time_elapsed++;                                     //Increment the elapsed time since peak
        }
    }

    return err;
}

/**
 * @brief adc_process_batt displays LED lights based on the battery adc value
 * @param value is the battery adc value
 */
void adc_process_batt(int value)
{
    if (value >= TWO_THIRDS_BATT_DEADBAND_UPPER)                            //If the battery level greater than is 2/3 full, display green
    {
        led_green(POWER_LED);
    }
    else if (value <= ONE_THIRD_BATT_DEADBAND_LOWER)                        //If battery is less than 1/3 full, display red
    {
        led_red (POWER_LED);
    }
    else if (value >= ONE_THIRD_BATT_DEADBAND_UPPER &&
                value <= TWO_THIRDS_BATT_DEADBAND_LOWER)                    //Else, display yellow
    {
        led_yellow (POWER_LED);
    }
}

int adc_process_bg_batt(int batt_value, int bg_value, int *bg)
{
    int err;

    adc_process_batt (batt_value);
    err = adc_process_bg (bg_value, bg);

    return err;
}

/**
 * @brief adc_idle_enter is a wrapper for bg_batt_adc_close
 */
int adc_idle_enter(void)
{
    int err;

    err = bg_batt_adc_close ();                                                  //Close adc and sample timer

    return err;
}

/**
 * @brief adc_idle_exit reinitializes the adc and sample timer
 */
int adc_idle_exit(void)
{
    int err;

    adc_update_bgm_state(BGM_WAITING_FOR_STRIP_STATE);

    err = bg_batt_adc_init ();                                                   //Init adc and sample timer

    return err;
}

/**
 * @brief bg_batt_adc_init is a wrapper for adc_init
 * @return 0 if success, else error code
 */
int bg_batt_adc_init(void)
{
    adc_sample_context_t *c = &adc_sample_context;

    c->state = ADC_BATT_SAMPLE;                                                 //Set the step to process only the battery level
    int err = adc_init (c);                                                     //Init adc and sample timer

    return err;
}

/**
 * @brief bg_batt_adc_close is a wrapper for adc_close
 * @return 0 if success, else error code
 */
int bg_batt_adc_close(void)
{
    adc_sample_context_t *c = &adc_sample_context;

    int err = adc_close (c);                                                    //Close adc and sample timer

    return err;
}

/**
 * @brief handle_strip_ins handles a strip insertion interrupt
 */
int handle_strip_ins()
{
    int err;
    adc_sample_context_t *c = &adc_sample_context;
    adc_peak_t *adc_p = &adc_peak;

    led_blue(STATUS_LED);
    adc_update_bgm_state(BGM_STRIP_INSERTED_STATE);
    err = tx_event_flags_set (c->res->event, DISPLAY_BP_EVENT, TX_OR);      //Tell display_thread_entry to display the Blood Prompt screen
    tx_thread_sleep (100);                                                  //Pause to debounce the strip insertion, else adc_track_peak will receive an impossibly high value
    adc_p->peak = 0;                                                        //Reset adc peak
    adc_p->peak_time_elapsed = 0;                                           //Reset elapsed time since peak
    c->state = ADC_BG_BATT_SAMPLE;                                           //Set the step to sample both the battery and blood glucose adc readings


#ifdef USE_DUMMY_DATA                                                       //If using dummy data
    i = 0;                                                                  //Set i to 0
    adc_idle_enter ();                                                      //Close adc and sample timer since data will be coming from test timer
    tx_thread_sleep (1);                                                    //Pause to let everything settle (probably unnecessary, but works so don't care)
    test_timer.p_api->open (test_timer.p_ctrl, test_timer.p_cfg);           //Open test timer
    test_timer.p_api->start (test_timer.p_ctrl);                            //Start test timer (could set to auto start in configuration)
#endif

    return err;
}

/**
 * @brief handle_strip_rem handles a strip removal interrupt
 **/
int handle_strip_rem()
{
    int err;
    adc_sample_context_t *c = &adc_sample_context;
    c->state = ADC_BATT_SAMPLE;                                              //Set the step to process only the battery level
    led_red (STATUS_LED);
    adc_update_bgm_state(BGM_WAITING_FOR_STRIP_STATE);
    err = tx_event_flags_set (c->res->event, DISPLAY_IS_EVENT, TX_OR);       //Tell display_thread_entry to display the Insert Strip Screen


#ifdef USE_DUMMY_DATA
    test_timer.p_api->stop (test_timer.p_ctrl);                              //If using dummy data, stop the test timer
#endif

    return err;
}
