/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "main_thread.h"
#include "adc_thread.h"
#include "adc_process.h"
#include "driver/led.h"
#include "display.h"
#include "driver/x_print.h"
#include "hal_data.h"
#include "fir_conf.h"
#include "config.h"
#include "events.h"
#include "util.h"
#include "application_dispatcher_thread.h"
#include "common.h"
#include "r_ioport.h"

extern healthcare_data g_healthcare_data;

void main_thread_entry(void);
int sys_idle_enter(void);
int sys_idle_exit(void);
//#define DO_X_PRINTF_RAW_TEST
//#define TEST_DISPLAY
//#define TEST_LEDS
//#define TEST_USBX
//#define TEST_RAW_USBX
//#define TEST_BG_ADC
//#define TEST_BATT_ADC
//#define TEST_CTSU
//#define TEST_LP
//#define TEST_TIMERS

#if defined (TEST_DISPLAY) || defined (TEST_LEDS) || defined (TEST_USBX) || defined (TEST_RAW_USBX) \
|| defined (TEST_BG_ADC ) || defined (TEST_BATT_ADC) || defined (TEST_CTSU) || defined (TEST_LP) || defined (TEST_TIMERS)
#define TEST
#endif

static void test(void);
#ifdef TEST
static void test_display(void);
static void test_leds(void);
static void test_usbx(void);
static void do_x_printf_raw_test(void);
static void test_bg_adc(void);
static void test_batt_adc(void);
static void test_lp(void);
static void test_timers(void);
#endif
static int board_init(void);

static void NMI_pin_init(void);

static void power_button_handler(bsp_grp_irq_t irq);

int handle_button_up(void);
int handle_button_down(void);

int check_strip (void);

/*
 * main_context_t contains necessary variables for the functioning of main_thread_entry
 */
typedef struct
{
    TX_EVENT_FLAGS_GROUP *event;
    ULONG actual_flags;
    bool strip_inserted;
    bool idle;
    int disp;
} main_context_t;

static main_context_t context =
{ .event = &adc_events,
  .idle = true
};

/* Main app thread entry function */
void main_thread_entry(void)
{
    main_context_t *c = &context;
    int err;

    board_init ();
    test ();


    while (err==0)                                                                          //Do while no error
    {
        err = tx_event_flags_get (c->event, PWR_EVENT_MASK | BTN_EVENT_MASK                 //Wait for an event dealing with
                                     | STRIP_EVENT_MASK,                                    //the power button, CTSU buttons or
                                     TX_OR_CLEAR, &c->actual_flags, TX_WAIT_FOREVER);       //strip interrupt


            if (BTN_UP_EVENT & c->actual_flags)
            {
                err = handle_button_up ();                                                        //CTSU button up handler
            }

            if (BTN_DOWN_EVENT & c->actual_flags)
            {
                err = handle_button_down ();                                                      //CTSU button down handler
            }

            if (STRIP_EVENT & c->actual_flags)
            {
                err = check_strip();
            }

            if (PWR_BTN_EVENT & c->actual_flags)
            {
                c->idle = (c->idle == true) ? false : true;                                 //Flip the bool value for if the board is idle
                err = (c->idle == true) ? sys_idle_enter () : sys_idle_exit ();                   //Run idle handler based on previous line
            }

    }
    led_white(STATUS_LED);                                                                  //Turn on LEDs if there is an error

    while(1)
    {
        tx_thread_sleep (1);
    }
}

/**
 * Callback for strip insert interrupt
 * @param p_args
 */
void strip_insert_callback(external_irq_callback_args_t * p_args)
{
    UNUSED(p_args);                                                                         //Eliminate compiler warnings
    main_context_t *c = &context;
    tx_event_flags_set (c->event, STRIP_EVENT, TX_OR);                                      //Send strip event flag to adc_process.c
}

/**
 * Callback for power button interrupt
 * @param irq
 */
static void power_button_handler(bsp_grp_irq_t irq)
{
    UNUSED(irq);
    tx_event_flags_set (&adc_events, PWR_BTN_EVENT, TX_OR);                                 //Send power button event flag to main_thread_entry
}

/**
 * Function to initialize the power button interrupt
 */
static void NMI_pin_init(void)
{
    R_ICU->NMICR_b.NMIMD = 0;   // falling edge
    R_ICU->NMICR_b.NFCLKSEL = 0x03; // PCLKB/64
    R_ICU->NMICR_b.NFLTEN = 1; // Digital filter is enabled
    //
    R_BSP_GroupIrqWrite (BSP_GRP_IRQ_NMI_PIN, power_button_handler);

    R_ICU->NMIER_b.NMIEN = 1; // NMI pin interrupt is enabled
}

/**
 * Initializes all components and interrupts on board
 */
static int board_init(void)
{
    int err=0;
    led_init ();                                                                            //Init LEDs
    x_print_njtag ();                                                                       //Init xprint
    NMI_pin_init ();                                                                        //Init power button interrupt
    if (err == 0) err = tx_event_flags_set (&adc_events, BOARD_INIT_EVENT, TX_OR);                              //Send board init event flag to display_thread_entry and adc_thread_entry
    tx_thread_sleep (10);

    return err;
}

/**
 * Closes components and interrupts except the power button interrupt
 */
int sys_idle_enter(void)
{
    int err;

    err = adc_idle_enter ();                                                                      //Close adc and sample timer
    if (err == 0) strip_ins_irq.p_api->close (strip_ins_irq.p_ctrl);                                      //Close strip sense interrupt
    display_timer.p_api->close (display_timer.p_ctrl);                                      //Close display timer
    led_all_off ();                                                                         //Turn off all LEDs
    display_str("   ");
    display_close ();                                                                       //Close LCD display
    // clear out all data
    tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
    memset(&g_healthcare_data, 0, sizeof(g_healthcare_data));
    tx_mutex_put(&g_data_mutex);
#ifdef USE_DUMMY_DATA
    test_timer.p_api->close (test_timer.p_ctrl);                                            //Close test timer if using dummy data
#endif

    return err;
}

int check_strip (void)
{
    // Create a variable for the pin level
    ioport_level_t pinlevel;
        // Create a variable for the returned error value.
    ssp_err_t err;

        // Read the level of pin on port 3 bit 9
    err = g_ioport.p_api->pinRead(IOPORT_PORT_00_PIN_08, &pinlevel);

    if (pinlevel == IOPORT_LEVEL_LOW && err == 0)
    {
        context.strip_inserted = true;
        err = handle_strip_ins ();
    }
    else
    {
        context.strip_inserted = false;
        err = handle_strip_rem ();
    }

    return err;
}
/**
 * Reopens components and interrupts, and resets various values to default states
 */
int sys_idle_exit(void)
{
    int err;
    main_context_t *c = &context;

    led_red(STATUS_LED);
    display_open ();                                                                        //Init LCD display
    err = adc_idle_exit ();                                                                       //Inits the adc and sample timer
    if (err == 0) strip_ins_irq.p_api->open (strip_ins_irq.p_ctrl, strip_ins_irq.p_cfg);                  //Init the strip sense IRQ
    if (err == 0) display_timer.p_api->open (display_timer.p_ctrl, display_timer.p_cfg);                  //Init the display timer
    if (err == 0) tx_event_flags_set (c->event, DISPLAY_IS_EVENT, TX_OR);                                 //Send display Insert Strip event to display_thread_entry
    if (err == 0) check_strip ();

    return err;
}

/**
 * CTSU up button handler
 */
int handle_button_up(void)
{
    return 0;
}

/**
 * CTSU down button handler
 */
int handle_button_down(void)
{
    return 0;
}

/**
 * test() runs the selected test functions on the board
 */
static void test(void)
{
#ifdef TEST
    test_display ();
    test_leds ();
    test_usbx ();
    do_x_printf_raw_test ();
    test_bg_adc ();
    test_batt_adc ();
    test_lp ();
    test_timers();
#endif
}
#ifdef TEST

static void test_display(void)
{
#ifdef TEST_DISPLAY
    float value = 0;

    display_open ();

    for (int i = 0; i < 100; i++)
    {
        display (value);

        value += 0.1;
        //
        tx_thread_sleep (4);
    }

#endif
}

static void test_leds(void)
{
#ifdef TEST_LEDS
    led_all_off ();
    for (int i = 0; i < 7; i++)
    {
        led_toggle (i);
        tx_thread_sleep (10);
        led_toggle (i);
    }
    led_all_off ();
    for (int i = 0; i < 7; i++)
    {
        led_on (i);
    }
#endif
}

static void test_usbx(void)
{
#ifdef TEST_USBX
    // some crazy long wait for the USB cable to be connected and the CONSOLE to get open
    tx_thread_sleep (100);

    // just show a hello message
    x_printff("MAIN THREAD Started!\n\r");
#endif
}

static void do_x_printf_raw_test(void)
{
#ifdef TEST_RAW_USBX
    // some crazy long wait for the USB cable to be connected and the CONSOLE to get open
    tx_thread_sleep (100);

    // just show a hello message
    x_printff ("MAIN THREAD Started!\n\r");

    while (1)
    {
        static int i = 0;
        led_toggle (LED_1);
        x_print_raw ((uint8_t *)&i, sizeof (i));
        x_print_raw_flush ();

        i++;
        tx_thread_sleep (1);
    }
#endif
}

static void test_bg_adc(void)
{
#ifdef TEST_BG_ADC
    adc_data_size_t strip_data;
    while (1)
    {
        if (stripInserted == true)
        {
            g_adc0.p_api->read (g_adc0.p_ctrl, ADC_REG_CHANNEL_3, &strip_data);
            x_printff("%d\n\r", strip_data);
        }
        tx_thread_sleep (1);
    }
#endif
}

static void test_batt_adc(void)
{
#ifdef TEST_BATT_ADC
    adc_data_size_t battery_data;
    int battery_percent;
    lpf *lp;
    lp = lpf_init ();
    while (1)
    {
        if (stripInserted == true)
        {
            g_adc0.p_api->read (g_adc0.p_ctrl, ADC_REG_CHANNEL_4, &battery_data);
            battery_percent = (int) ((double) lpf_process (lp, battery_data) - DEAD_BAT) / (FULL_BAT - DEAD_BAT) * 100;
            battery_percent = (int) ((double) battery_data - DEAD_BAT) / (FULL_BAT - DEAD_BAT) * 100;
            x_printff("%d\n\r", battery_percent);
        }
        tx_thread_sleep (1);
    }
#endif
}
static void test_lp(void)
{
#ifdef TEST_LP
    lpf *lp;
    int i, lpfo;
    lp = lpf_init ();
    for (i = 0; i < 2002; i++)
    {
        lpfo = lpf_process (lp, inputs_int[i]);
        //x_printff("%d\n\r", lpfo);
    }
#endif
}

static void test_timers(void)
{
#ifdef TEST_TIMERS
    sample_timer_open (&adc_mcu_res);
    settle_timer_open (&adc_mcu_res);
    settle_timer_start (&adc_mcu_res);
#endif
}
#endif
