/*
  ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * adc_process.c
 *
 *  Created on: Jul 29, 2016
 *      Author: mitsumono
 ****************************************************************************************
 */

#include "adc_process.h"
#include "driver/x_print.h"
#include "application_dispatcher_thread.h"
#include "common.h"
#include "display.h"

#define RETRY_BPM_LIMIT     10
// 2 ms tick rate or 500 ticks per second
#define BP_TIMEOUT_LIMIT    45000   // 90 seconds

#define CUFF_AVG_K  0.1

// mfr - for internal testing
//static int s_cuff_adc = 0;
//static int s_op_amp_adc = 0;

extern healthcare_data g_healthcare_data;

adc_peak_t adc_peak;

results_t results;

long int time_elapsed;

void adc_update_bpm_state(adc_state_t state);

void adc_update_bpm_state(adc_state_t state)
{
    // update values
    tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
    g_healthcare_data.bpm_state = (uint16_t) state;
    tx_mutex_put(&g_data_mutex);
}

/**
 * @brief adc_process_bg process cuff adc values coming from adc_thread_entry
 * @param adc_bp_val is the cuff adc value
 * @param bp_val is a pointer to the location of the filtered adc value, used for calculating the pressures, bpm, and for debugging purposes in adc_thread_entry
 * @param state is the current state of the processing
 */
void adc_process_bp(adc_state_t *state, results_t *r)
{
    adc_state_t prev_state = INIT_PROCESS;
    adc_sample_context_t *c = &adc_sample_context;
    int measure_complete;
    static bool avg_first_time = true;

    time_elapsed++;                                                                                     //Increment the time stamp counter
    // running average on cuff pressure
    r->bp_adc_cuff_lp = running_avg(CUFF_AVG_K, r->bp_adc_cuff_lp, r->bp_adc_cuff_val, avg_first_time);
    avg_first_time = false;
    filter_adc (r->bp_adc_opamp_val, &(r->bp_adc_opamp_lp), &(r->bp_value));                                        //Filter the adc cuff value to remove noise and the pressure increase from the pump
    set_variable_threshold (r->bp_value, &adc_peak);                                                    //Set the variable threshold by low passing the filtered adc peaks

    // timeout if taking too long for measurements or pressure too high for demo
    if ((r->bp_adc_cuff_lp > ONE_SIXTY_MMGH) || (c->timeout_tick++ > BP_TIMEOUT_LIMIT))                                                                      //If we reach this level, we have probably
    {                                                                                                   //failed to find correct results
        *state = MEASUREMENT_FAILED;
    }

    switch (*state)
    {
        case INIT_PROCESS:                                                                             //INIT_PROCESS
            prev_state = INIT_PROCESS;
            adc_update_bpm_state(INIT_PROCESS);
            pwm_start ();                                                                               //Turn on the pump
            valve_close ();                                                                             //Close the valve
            tx_event_flags_set (c->res->event, DISPLAY_WAIT_EVENT, TX_OR);                              //Tell display_thread_entry to wait for the result
            led_yellow (STATUS_LED);                                                                    //Change LED to indicate status
            *state = FIND_DIASTOLIC_PRESSURE;                                                           //Switch to the diastolic pressure state
            c->retry_count = 0;
            c->timeout_tick = 0;
            avg_first_time = true;
        break;

        case FIND_DIASTOLIC_PRESSURE:                                                                   //FIND_DIASTOLIC_PRESSURE
            if (*state != prev_state)
            {
                adc_update_bpm_state(FIND_DIASTOLIC_PRESSURE);
                display_str("dI ");
                prev_state = *state;
            }

            measure_complete = find_diastolic_pressure (r->bp_value, r->bp_adc_opamp_lp, r->bp_adc_cuff_lp, time_elapsed, &adc_peak);   //Analyze received data
            if (measure_complete == MEASUREMENT_SUCCESS)
            {
                pwm_stop ();                                                                            //Stop the pump for measuring bpm
                led_blue (STATUS_LED);                                                                  //Change LED to indicate status
                *state = FIND_BPM;                                                                      //Switch to find bpm pressure state
            }
        break;

        case FIND_BPM:                                                                                  //FIND BPM
            if (*state != prev_state)
            {
                adc_update_bpm_state(FIND_BPM);
                display_str("bR ");
                prev_state = *state;
            }
            measure_complete = find_bpm (r->bp_value, r->bp_adc_opamp_lp, time_elapsed, &adc_peak);                  //Analyze received data
            if (measure_complete == MEASUREMENT_SUCCESS)
            {
                pwm_start ();                                                                           //Start PWM to find Systolic pressure
                led_green (STATUS_LED);                                                                 //Switch LED to green
                tx_thread_sleep (100);                                                                  //Sleep the thread to give time for pressure to increase, else we
                *state = FIND_SYSTOLIC_PRESSURE;                                                        //may immediately set the systolic pressure incorrectly
            }
            if (measure_complete == MEASURE_FAILED)                                                     //If measurement failed, it means we incorrectly recorded
            {                                                                                           //diastolic pressure. Return and try again
                pwm_start ();                                                                           //Start the PWM to find diastolic pressure
                led_yellow (STATUS_LED);                                                                //Switch the LED back
                *state = FIND_DIASTOLIC_PRESSURE;                                                       //Switch state
                c->retry_count++;
                if (c->retry_count > RETRY_BPM_LIMIT)
                {
                    *state = MEASUREMENT_FAILED;
                }
            }
        break;

        case FIND_SYSTOLIC_PRESSURE:                                                                    //FIND_SYSTOLIC_PRESSURE
            if (*state != prev_state)
            {
                adc_update_bpm_state(FIND_SYSTOLIC_PRESSURE);
                display_str("St ");
                prev_state = *state;
            }
            measure_complete = find_systolic_pressure (r->bp_adc_cuff_lp, &adc_peak);                           //Analyze received data
            if (measure_complete == MEASUREMENT_SUCCESS)
            {
                pwm_stop ();                                                                            //Stop the PWM
                valve_open ();                                                                          //Open the valve
                *state = MEASUREMENT_COMPLETE;
            }
        break;

        case MEASUREMENT_COMPLETE:                                                                      //MEASUREMENT_COMPLETE
            //Handle measurement complete state
            adc_update_bpm_state(MEASUREMENT_COMPLETE);
            c->retry_count = 0;
            c->timeout_tick = 0;
        break;

        case MEASUREMENT_FAILED:                                                                        //MEASUREMENT_FAILED
            //Handle measurement failed state
            adc_update_bpm_state(MEASUREMENT_FAILED);
            led_red(STATUS_LED);
            pwm_stop();                                                                                    //Stop the pump
            valve_open();                                                                                  //Open the valve
            tx_event_flags_set (c->res->event, DISPLAY_FAILED_EVENT, TX_OR);                                //Tell the display_entry_thread to display the failed screen
            // reset timeouts and counters
            c->retry_count = 0;
            c->timeout_tick = 0;
            avg_first_time = true;
        break;
        default:
        break;
    }
}

/**
 * @brief find_diastolic_pressure analyzes the incoming data for peaks and looks for a heart beat trend to locate the diastolic pressure
 * @param value is the incoming filtered adc data
 * @param low_pass is the incoming adc data that has been passed through an lp filter
 * @param timestamp is the current timestamp
 * @param adc_p is the adc_peak_t struct using the function
 * @return
 */
adc_measurement_return find_diastolic_pressure(int value, int low_pass, int pressure, long int timestamp, adc_peak_t *adc_p)
{
    adc_peak_t *c = adc_p;
    results_t *r = &results;
    // These variables are used to increase clarity and code below
    int *current_peak = &c->curr_peak;
    int *current_peak_pulse = &c->peak_pulses[c->peak_buff_head];
    long int *current_peak_timestamp = &c->peak_timestamps[c->peak_buff_head];
    int *current_peak_threshold = &c->peak_thresholds[c->peak_buff_head];
    long int *previous_peak_timestamp = &c->peak_timestamps[
            (c->peak_buff_head == 0) ? TS_BUFF_SIZE - 1 : c->peak_buff_head - 1];
    int *current_diastolic_pressure = &c->diastolic_pressures[c->peak_buff_head];
    long int *bpm = &c->beats_per_minute;
    int *dia_pressure_res = &r->diastolic_pressure;
    long int peak_period = 0;
    peak_error_t peak_validate;

    if (pressure < FOURTY_MMGH)                                                      //If the current cuff pressure is less than specified mmhg skip analysis because
        return CONTINUE_MEASUREMENT;                                                //blood pressure for this demo should never be this low

    if (value < c->variable_threshold && *current_peak < c->variable_threshold)     //If value and current_peak are less than the threshold, continue receiving data
        return CONTINUE_MEASUREMENT;

    if (value > *current_peak)                                                      //If the current adc value is greater than the saved value...
    {
        *current_peak = value;                                                      //Update the peak value
        *current_peak_pulse = low_pass;                                           //Record the low passed pressure of the peak
        *current_peak_timestamp = timestamp;                                        //Reset the elapsed time since peak
        *current_diastolic_pressure = pressure;
    }

    if (value >= c->variable_threshold)                                              //If incoming value is greater than the variable threshold
        return CONTINUE_MEASUREMENT;                                                //we are still in a peak, continue measuring

    //If value is less than variable threshold and we have recorded a local peak, process this peak

    *current_peak_threshold = c->variable_threshold;                                //Record the variable threshold during this peak (will be used to find systolic pressure)
    *current_peak = 0;                                                              //Reset peak value

    if (c->peak_buff_head != c->peak_buff_start)                                    //If this isn't our first peak, calculate bpm and peak_period
    {
        peak_period = *current_peak_timestamp - *previous_peak_timestamp;           //Find the time between peaks
        *bpm = *bpm + peak_period;                                                  //Accumulate the time between peaks in the bpm counter
        if (c->peak_buff_head == (c->peak_buff_start + 1) % TS_BUFF_SIZE)           //If this is the first time we have calculated peak period, record it
            c->tracked_bpm = peak_period;                                           //Use the first time between peaks for comparison against later periods
    }

    peak_validate = validate_peak (peak_period, c);                                 //Compare peak data against criteria to see if it is following a heart beat trend
    switch (peak_validate)
    {
        case INCONSISTANT_PEAKS:                                                    //If the peak periods vary too much to be a heartbeat...
            c->peak_buff_start = (c->peak_buff_start + 1) % TS_BUFF_SIZE;           //Increment the start of the heart beat buffer by one to throw away oldest peak
            c->tracked_bpm = peak_period;                                           //Set the tracked bpm to the current peak period
            *bpm = c->tracked_bpm;                                                  //Set the accumulated bpm to the peak period
            c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;             //Increment the head of the peak buffer by one
        break;
        case PEAK_PERIOD_OUT_OF_RANGE:                                              //If the peak period is not in the range of a heart beat, we completely throw out the buffer
            c->peak_buff_start = c->peak_buff_head;                                 //Set the buffer start to the buffer head
            *bpm = 0;                                                               //Reset accumulated bpm
            c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;             //Increment the buffer head by one
        break;
        case MISSED_PEAK:                                                           //The code currently allows us to miss one peak. This can be removed from validate_peak to make a more strict algorithm
            c->missed_peak_count++;                                                 //Record the missed peak
            c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;             //Increment buffer head
        break;
        case FIRST_PEAK:                                                            //If this is our first peak...
            //fallthrough
        case PEAK_VALID:                                                            //..or the peak is valid
            c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;             //Increment the buffer head
        break;
    }
    if (((c->peak_buff_head - c->peak_buff_start) == 3) ||                              //If we have recorded 3 valid peaks, set the diastolic pressure
            ((c->peak_buff_head - c->peak_buff_start) == -7))                           //and tell adc_process_bp to switch to finding beats per minute
    {
        *dia_pressure_res = convert_pressure (c->diastolic_pressures[c->peak_buff_start]);   //the diastolic pressure
        return MEASUREMENT_SUCCESS;
    }
    return CONTINUE_MEASUREMENT;                                                    //Included to make the compiler shut up. Should never run.
}

/**
 * @brief find_bpm records 7 more peaks after find_diastolic_pressure for a total of 10 peaks, to validate the heart beat and average the beats per minute
 * @param value is the filtered cuff adc value
 * @param low_pass is the low passed cuff adc value
 * @param timestamp is the current timestamp
 * @param adc_p is the peak struct we are using for adc_process
 * @return returns success or failure of find_bpm
 */
adc_measurement_return find_bpm(int value, int low_pass, long int timestamp, adc_peak_t *adc_p)
{
    adc_peak_t *c = adc_p;
    results_t *r = &results;
    int *current_peak = &c->curr_peak;
    int *current_peak_pulse = &c->peak_pulses[c->peak_buff_head];
    long int *current_peak_timestamp = &c->peak_timestamps[c->peak_buff_head];
    long int *previous_peak_timestamp = &c->peak_timestamps[
            (c->peak_buff_head == 0) ? TS_BUFF_SIZE - 1 : c->peak_buff_head - 1];
    long int *bpm = &c->beats_per_minute;
    int *bpm_res = &r->beats_per_minute;
    long int peak_period;
    peak_error_t peak_validate;

    if (value < c->variable_threshold && *current_peak < c->variable_threshold)                             //If value and current_peak are less than the threshold, we aren't in a peak, check for time out
    {
        if (timestamp - *previous_peak_timestamp > (c->tracked_bpm * 2 + QUARTER_SECOND))                   //If we haven't received a peak in greater than 2 times our expected bpm, report failure
        {
            c->peak_buff_start = c->peak_buff_head;                                                         //Throw away peak buffer
            *bpm = 0;                                                                                       //Reset accumulated bpm
            return MEASURE_FAILED;                                                                          //Report failure
        }
        else
            return CONTINUE_MEASUREMENT;                                                                    //Else if we haven't timed out, continue measurement
    }

    if (value > *current_peak)                                                                              //If the current adc value is greater than the saved value...
    {
        *current_peak = value;                                                                              //Update the peak value
        *current_peak_pulse = low_pass;                                                                   //Update the adc cuff low pass filter
        *current_peak_timestamp = timestamp;                                                                //Reset the elapsed time since peak
    }

    if (value >= c->variable_threshold)                                                                      //If incoming value is greater than the variable threshold
        return CONTINUE_MEASUREMENT;                                                                        //we are still in a peak, continue measuring

    //If value is less than variable threshold and we have recorded a local peak, process this peak

    *current_peak = 0;                                                                                      //Reset peak tracking
    peak_period = *current_peak_timestamp - *previous_peak_timestamp;                                       //Find the time between peaks
    *bpm = *bpm + peak_period;                                                                              //Accumulate the time between peaks in the bpm counter
    peak_validate = validate_peak (peak_period, c);                                                         //Validate the peak
    switch (peak_validate)
    {
        case INCONSISTANT_PEAKS:                                                                            //If the peak is inconsistant with other peaks...
            if (c->peak_failed_count == ALLOWED_PEAK_FAILS)                                                 //If we have exceeded the maximum number of failed peaks (currently 0)
            {
                c->peak_failed_count = 0;                                                                   //Reset failed peak count
                c->missed_peak_count = 0;                                                                   //Reset mixed peak count
                c->peak_buff_start = (c->peak_buff_head == 0) ? TS_BUFF_SIZE - 1 : c->peak_buff_head - 1;   //Update the buffer start to one entry behind the head
                c->tracked_bpm = peak_period;                                                               //Set the new tracked peak period to the current peak period
                *bpm = c->tracked_bpm;                                                                      //Reset the accumulated beats per minute to the current peak period
                c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;                                 //Increment the buffer head
                return MEASURE_FAILED;                                                                      //Return failed
            }
            else
            {
                c->peak_failed_count++;                                                                     //If we haven't exceeded the number of failed peaks, increment
            }
        break;
        case PEAK_PERIOD_OUT_OF_RANGE:                                                                      //If we are out of range...
            if (c->peak_failed_count == ALLOWED_PEAK_FAILS)
            {
                c->peak_failed_count = 0;                                                                   //Reset failed peak count
                c->missed_peak_count = 0;                                                                   //Reset mixed peak count
                c->peak_buff_start = c->peak_buff_head;                                                     //Throw away buffer contents except for the current peak
                *bpm = 0;                                                                                   //Reset accumulated beats per minute
                c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;                                 //Increment the buffer head
                return MEASURE_FAILED;                                                                      //Return failed
            }
            else
            {
                c->peak_failed_count++;                                                                     //If we haven't exceeded the number of failed peaks, increment
            }
        break;
        case MISSED_PEAK:                                                                                   //The code currently allows us to miss one peak. This can be removed from validate_peak to make a more strict algorithm
            c->missed_peak_count++;                                                                         //Record the missed peak
            c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;                                     //Increment buffer head
        break;
        case FIRST_PEAK:                                                                                    //If this is our first peak...
            //fallthrough
        case PEAK_VALID:                                                                                    //..or the peak is valid
            c->peak_buff_head = (c->peak_buff_head + 1) % TS_BUFF_SIZE;                                     //Increment the buffer head
        break;
    }
    if (c->peak_buff_head == c->peak_buff_start)                                                            //If we have wrapped around the buffer...
    {

        c->tracked_bpm = *bpm / ((TS_BUFF_SIZE - 1) + c->missed_peak_count);                                //Calculate average period between peaks
        *bpm_res = 1000 / (c->tracked_bpm * 2 / 60);                                                        //Calculate average bpm, store to results
        //*bpm_res = (int) 1 / ((double) (c->tracked_bpm) * 2 / 1000) * 60;
        return MEASUREMENT_SUCCESS;
    }
    return CONTINUE_MEASUREMENT;
}

/**
 * @brief find_systolic_pressure observes the variable threshold and returns the pressure at which it falls below a certain threshold as the systolic pressure
 * @param low_pass is the filtered cuff adc pressure
 * @param adc_p is the peak struct being used in adc_process
 * @return returns the success of the measurement
 */
adc_measurement_return find_systolic_pressure(int pressure, adc_peak_t *adc_p)
{
    adc_peak_t *c = adc_p;
    results_t *r = &results;
    int *lower_threshold = &c->peak_thresholds[c->peak_buff_start];
    int *sys_pressure_res = &r->systolic_pressure;

    if (c->variable_threshold >= c->max_threshold)                                                           //First we find the maximum variable threshold number
    {
        c->max_threshold = c->variable_threshold;
    }
    else
    {
        if (c->variable_threshold < *lower_threshold + (c->max_threshold - *lower_threshold) / 8)              //When the variable threshold falls to a certain level
        {                                                                                                       //designated somewhere between the threshold at the diastolic pressure
            *sys_pressure_res = convert_pressure (pressure);                                                     //and the maximum threshold, return this as the systolic pressure
            return MEASUREMENT_SUCCESS;
        }
    }

    return CONTINUE_MEASUREMENT;
}

/**
 * @brief convert_pressure converts the provided adc value to a value of units MMGH
 * @param adc_pressure is the low passed cuff adc value
 * @return
 */
int convert_pressure(int adc_pressure)
{
    return 160 * (adc_pressure - ZERO_MMGH) / (ONE_SIXTY_MMGH - ZERO_MMGH);
}

/**
 * @brief set_variable_threshold adjusts the threshold based on new data
 * @param value is the filtered adc cuff value
 * @param adc_p is the peak struct being used by adc_process
 */
void set_variable_threshold(int low_pass_val, adc_peak_t *adc_p)
{
    adc_peak_t *c = adc_p;

    if (low_pass_val >= c->local_peak)                                                                  //If value is greater than the current peak, update
    {
        c->local_peak = low_pass_val;
    }
    else if (low_pass_val < 0)                                                                          //If value is less than zero, update variable threshold
    {
        if (c->local_peak > LOWER_NOISE_THRESHOLD && c->local_peak < UPPER_NOISE_THRESHOLD)             //These constants can be used to control how much noise affects the threshold
        {
            update_threshold (&c->variable_threshold, c->local_peak, &c->previous_filtered_value, 0.2); //Update the threshold (function is in filter.c since it is an lp filter)
        }
        c->local_peak = 0;                                                                              //Reset the local peak
    }
}

/**
 * @brief validate_peak compares a peak to a series of criterion to decide if it follows a heart beat trend
 * @param peak_period is the length between the current peak and the last peak
 * @param adc_p is the peak struct being used in adc_process
 * @return returns a peak_error_t value indicating if the peak is valid or not, and why it isn't valid
 */
peak_error_t validate_peak(int peak_period, adc_peak_t *adc_p)
{
    adc_peak_t *c = adc_p;

    if (peak_period == 0)                                                                                   //If this is our first peak, return immediately, nothing to check
        return FIRST_PEAK;

    if ((peak_period > THIRTY_BPM) || (peak_period < ONE_HUNDRED_BPM))                                          //If the beats per minute is less than 30 beats per minute or greater than 100,
        return PEAK_PERIOD_OUT_OF_RANGE;                                                                    //return failure (can be adjusted)

    if (peak_period <= c->tracked_bpm - QUARTER_SECOND ||                                                   //If the peak period is less than the tracked peak take a quarter second or...
            peak_period >= c->tracked_bpm * 2 + QUARTER_SECOND ||                                           //If the peak period is greater than the tracked peak times two given a quarter second or...
            (peak_period >= c->tracked_bpm + QUARTER_SECOND && peak_period <= c->tracked_bpm * 2 - QUARTER_SECOND)) //If the peak is between a peak given a quarter second and two times a peak take a quarter second...
        return INCONSISTANT_PEAKS;                                                                          //Return that this peak period is inconsistent with the period of other peaks
                                                                                                            //Note that this allows us to miss one beat if it falls under our variable threshold
    if (peak_period >= c->tracked_bpm * 2 - QUARTER_SECOND)                                                 //If the peak isn't inconsistent, but we missed a peak, return missed peak
        return MISSED_PEAK;

    return PEAK_VALID;                                                                                      //Else, return peak valid
}

/**
 * @brief adc_process_batt displays LED lights based on the battery adc value
 * @param value is the battery adc value
 */
void adc_process_batt(int value)
{

    if (value >= TWO_THIRDS_BATT_DEADBAND_UPPER)                                                //If the battery level greater than is 2/3 full, display green
    {
        led_green (POWER_LED);
    }
    else if (value <= ONE_THIRD_BATT_DEADBAND_LOWER)                                            //If battery is less than 1/3 full, display red
    {
        led_red (POWER_LED);
    }
    else if (value >= ONE_THIRD_BATT_DEADBAND_UPPER && value <= TWO_THIRDS_BATT_DEADBAND_LOWER) //Else, display yellow
    {
        led_yellow (POWER_LED);
    }
}

/**
 * @brief adc_process_bp_batt runs the processes for both the battery and cuff
 * @param batt_val is the battery adc value
 * @param adc_bp_val is the cuff adc value
 * @param bp_val is a pointer to the location of the filtered adc value, used for calculating the pressures, bpm, and for debugging purposes in adc_thread_entry
 * @param state is the current state of the processing
 */
void adc_process_bp_batt(adc_state_t *state, results_t *r)
{
    adc_process_batt (r->batt_value);
    adc_process_bp (state, r);
}


/**
 * @brief adc_idle_enter is a wrapper for bg_batt_adc_close
 */
int adc_idle_enter(void)
{
    return bp_batt_adc_close ();    //Close adc and sample timer
}

/**
 * @brief adc_idle_exit reinitializes the adc and sample timer
 */
int adc_idle_exit(void)
{
    return bp_batt_adc_init ();     //Init adc and sample timer
}

/**
 * @brief bg_batt_adc_init is a wrapper for adc_init
 * @return 0 if success, else error code
 */
int bp_batt_adc_init(void)
{
    int err = 0;
    adc_sample_context_t *c = &adc_sample_context;
    c->mode = ADC_BP_BATT_SAMPLE;   //Set the step to process both the cuff and battery for now until we have button support
    c->state = INIT_PROCESS;        //Immediately start the measuring process
#if !defined(USE_DUMMY_DATA)
    err = adc_init (c);             //Init adc and sample timer
#endif
    return err;
}

/**
 * @brief bg_batt_adc_close is a wrapper for adc_close
 * @return 0 if success, else error code
 */
int bp_batt_adc_close(void)
{
    adc_sample_context_t *c = &adc_sample_context;
    pwm_stop ();
    valve_open ();
    int err = adc_close (c);        //Close adc and sample timer

    return err;
}
