#include "main_thread.h"
#include "hal_data.h"
#include "driver/led.h"
#include "display.h"
#include "driver/x_print.h"
#include "events.h"
#include "util.h"
#include "adc_process.h"
#include "hal/adc_ssp.h"
#include "adc.h"
#include "application_dispatcher_thread.h"
#include "common.h"

extern healthcare_data g_healthcare_data;

void main_thread_entry(void);

static void board_init(void);

static void NMI_pin_init(void);

static void power_button_handler(bsp_grp_irq_t irq);

int sys_idle_enter(void);
int sys_idle_exit(void);

void handle_button_up(void);
void handle_button_down(void);

void test_adc (void);

/*
 * main_context_t contains necessary variables for the functioning of main_thread_entry
 */
typedef struct
{
    TX_EVENT_FLAGS_GROUP *event;
    ULONG actual_flags;
    bool idle;
    int disp;
} main_context_t;

static main_context_t context =
{ .event = &adc_events,
  .idle = true
};

/* Main Thread entry function */
void main_thread_entry(void)
{
    main_context_t *c = &context;
    int err;

    pump_off ();
    board_init ();

    // initial state should be idle; waiting for button press
    sys_idle_enter();

    while (err==0)                                                                          //Do while no error
    {
        err = tx_event_flags_get (c->event, PWR_EVENT_MASK | BTN_EVENT_MASK,                //Wait for an event dealing with
                                     TX_OR_CLEAR, &c->actual_flags, TX_WAIT_FOREVER);       //the power button or CTSU buttons

            if (BTN_UP_EVENT & c->actual_flags)
            {
                handle_button_up ();                                                        //CTSU button up handler
            }

            if (BTN_DOWN_EVENT & c->actual_flags)
            {
                handle_button_down ();                                                      //CTSU button down handler
            }

            if (PWR_BTN_EVENT & c->actual_flags)
            {
                c->idle = (c->idle == true) ? false : true;                                 //Flip the bool value for if the board is idle
                (c->idle == true) ? sys_idle_enter () : sys_idle_exit ();                   //Run idle handler based on previous line
            }

    }
    led_white(STATUS_LED);                                                                  //Turn on the LED if there is an error

    // not suppose to be here
    while (1)
    {
        tx_thread_sleep (1);
    }
}


void g_button_framework_user_callback(sf_touch_ctsu_button_callback_args_t * p_args)
{
    UNUSED (p_args);
}

static void board_init (void)
{
    int err=0;
    led_init ();                                                                                                //Init LEDs
    x_print_njtag ();                                                                                           //Init xprint
    NMI_pin_init ();                                                                                            //Init power button interrupt                                                                    //Init ADC and sample timer
#if defined(USE_DUMMY_DATA)
    test_timer.p_api->open (test_timer.p_ctrl, test_timer.p_cfg);                                             //Used when using dummy data
#endif
    if (err == 0) err = tx_event_flags_set (&adc_events, BOARD_INIT_EVENT_ADC, TX_OR);                          //Send board init event flag to display_thread_entry and adc_thread_entry
    if (err == 0) err = tx_event_flags_set (&adc_events, BOARD_INIT_EVENT_DISP, TX_OR);                         //Send board init event flag to display_thread_entry and adc_thread_entry
    //if (err == 0) err = tx_event_flags_set (&adc_events, DISPLAY_IDLE_EVENT, TX_OR);                          //Uncomment once CTSU has been added and we are not immediately running the measurements
}

/**
 * CTSU up button handler
 */
void handle_button_up(void)
{

}

/**
 * CTSU down button handler
 */
void handle_button_down(void)
{

}

/**
 * Closes components and interrupts except the power button interrupt
 */
int sys_idle_enter(void)
{
    int err;
    err = display_timer.p_api->close (display_timer.p_ctrl);                                                          //Close display timer
    display_close ();
    err = adc_idle_enter ();                                                                                    //Close adc and sample timer
    led_all_off ();                                                                           //Turn off all LEDs
    // clear out all data
    tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
    memset(&g_healthcare_data, 0, sizeof(g_healthcare_data));
    tx_mutex_put(&g_data_mutex);
    #if defined(USE_DUMMY_DATA)
    err = test_timer.p_api->close (test_timer.p_ctrl);                                        //Close test timer if using dummy data
#endif


    return err;
}

/**
 * Reopens components and interrupts, and resets various values to default states
 */
int sys_idle_exit(void)
{
    int err;
    main_context_t *c = &context;

    display_open ();                                                                                            //Init LCD display
    err = adc_idle_exit ();                                                                                     //Inits the adc and sample timer
    if (err == 0) display_timer.p_api->open (display_timer.p_ctrl, display_timer.p_cfg);                        //Init the display timer
    if (err == 0) tx_event_flags_set (c->event, DISPLAY_IDLE_EVENT, TX_OR);                                     //Send display nothing event to display_thread_entry

    return err;
}

/**
 * Callback for power button interrupt
 * @param irq
 */
static void power_button_handler(bsp_grp_irq_t irq)
{
    UNUSED(irq);
    tx_event_flags_set (&adc_events, PWR_BTN_EVENT, TX_OR);                                                     //Send power button event flag to main_thread_entry
}

/**
 * Function to initialize the power button interrupt
 */
static void NMI_pin_init(void)
{
    R_ICU->NMICR_b.NMIMD = 0;   // falling edge
    R_ICU->NMICR_b.NFCLKSEL = 0x03; // PCLKB/64
    R_ICU->NMICR_b.NFLTEN = 1; // Digital filter is enabled
    //
    R_BSP_GroupIrqWrite (BSP_GRP_IRQ_NMI_PIN, power_button_handler);

    R_ICU->NMIER_b.NMIEN = 1; // NMI pin interrupt is enabled
}


void test_adc (void)
{
    x_print_njtag ();
    //tx_thread_sleep(1000);
    x_printff("Hello World\n\r");
    adc_init(&adc_sample_context);
    adc_sample_context.mode = ADC_TEST;
    while (1)
    {
        tx_thread_sleep(1);
    }
}
