//
//  X1Services.m
//  BleDemo
//
//  Created by James Roth on 6/12/14.
//  Copyright (c) 2014 Xaplos. All rights reserved.
//

#import <CoreBluetooth/CBUUID.h>
#import <CoreBluetooth/CBService.h>
#import <CoreBluetooth/CBCharacteristic.h>

#import "BleDemoServices.h"
#import "SensorData.h"
#import "HealthData.h"


#define PERROR(e)                                                           \
do {                                                                        \
    if (e) {                                                                \
        NSLog(@"%s error %@", __FUNCTION__, [e localizedDescription]);      \
        [self.notificationDelegate errorOccured:[e localizedDescription]];  \
    }                                                                       \
} while (0)

//
// A container for a charactersitic that will be streamed with notifications
//
@interface X1Streamable : NSObject

@property (nonatomic, retain) NSString *identifier;
@property (nonatomic, retain) CBUUID *characteristicUUID;
@property (nonatomic, retain) CBCharacteristic *characteristic;
@property (nonatomic, retain) id<X1CharacteristicParser>parser;
@property (nonatomic) BOOL readInitialValue;

@end

@implementation X1Streamable

- (id)initWithCharacteristic:(NSString *)charactersticString
                      parser:(id<X1CharacteristicParser>)parser readInitialValue:(BOOL)readInitialValue {
    if (self = [super init]) {
        self.identifier = charactersticString;
        self.characteristicUUID = [CBUUID UUIDWithString:charactersticString];
        self.characteristic = nil;
        self.parser = parser;
        self.readInitialValue = readInitialValue;
    }
    
    return self;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"<X1Streamable: %@ %@>",
            self.characteristicUUID, self.characteristic];
}

@end

@implementation X1Services


- (id)initWithPeripheral:(CBPeripheral *)p {
    if (self = [super init]) {
        NSLog(@"BleDemoServices init");
        p.delegate = self;
        self.peripheral = p;
        
        // set service to discover
        [self discoverWithService:HC_DEMO_SERVICE];
        
        X1Streamable *activity = [[X1Streamable alloc] initWithCharacteristic:HC_ACTIVITY_SENSOR
                                                            parser:[[SensorData alloc] init]
                                                            readInitialValue:false];
        
        X1Streamable *health = [[X1Streamable alloc] initWithCharacteristic:HC_HEALTHCARE_SENSOR
                                                            parser:[[HealthData alloc] init]
                                                            readInitialValue:false];
        
        self.streamables = [NSDictionary dictionaryWithObjectsAndKeys:
                            activity, activity.characteristicUUID,
                            health, health.characteristicUUID,
                            nil];
    }
    
    return self;
}

- (void)dealloc {
    // Must remove observer; otherwise the Notification Center will
    // continue to try and send notification objects to the deallocated
    // object
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)discover {
    NSLog(@"calling discoverServices");
    // add small delay to allow target time to process message
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(250 * NSEC_PER_MSEC)), dispatch_get_main_queue(), ^{
    [self.peripheral discoverServices:nil];
});
    //[self.peripheral discoverServices:nil];
}

- (void)discoverWithService:(NSString *)serviceUUIDString {
    self.serviceUUID = [CBUUID UUIDWithString:serviceUUIDString];
}

- (void)setNotifications:(BOOL)enable {
    for (X1Streamable *streamable in [self.streamables allValues]) {
        NSLog(@"enable %@", streamable);
        
        if (streamable.characteristic) {
            [self.peripheral setNotifyValue:enable forCharacteristic:streamable.characteristic];
        }
        else {
            NSLog(@"no characterstic to enable");
        }
    }
}

- (BOOL)discoveryComplete {
    for (X1Streamable *streamable in [self.streamables allValues]) {
        if (!streamable.characteristic) {
            return FALSE;
        }
    }
    
    return TRUE;
}

#pragma mark CBPeripheralDelegate

- (void)                  peripheral:(CBPeripheral *)peripheral
didDiscoverCharacteristicsForService:(CBService *)service
                               error:(NSError *)error
{
    PERROR(error);
    NSLog(@"%@ characteristics %@", service.UUID, service.characteristics);
    
    for (CBCharacteristic *characteristic in service.characteristics) {
        X1Streamable *streamable = [self.streamables objectForKey:characteristic.UUID];
        
        if (streamable && [streamable.characteristicUUID.data isEqualToData:characteristic.UUID.data]) {
            streamable.characteristic = characteristic;
            // force an initial read only on "real data"
            if (streamable.readInitialValue) {
                [peripheral readValueForCharacteristic:characteristic];
            }
        }
    }
 
    if ([self discoveryComplete]) {
        [self setNotifications:YES];
    }
}

- (void) peripheral:(CBPeripheral *)peripheral
didDiscoverServices:(NSError *)error
{
    PERROR(error);
    NSLog(@"services:");
    
    for (CBService *service in peripheral.services) {
        NSLog(@"%@", service);
        [peripheral discoverCharacteristics:nil forService:service];
    }
}

- (void)                         peripheral:(CBPeripheral *)peripheral
didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic
                                      error:(NSError *)error
{
    PERROR(error);
    NSLog(@"notify updated %@", characteristic.UUID);
}

- (void)             peripheral:(CBPeripheral *)peripheral
didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic
                          error:(NSError *)error
{
    PERROR(error);
    X1Streamable *streamable = [self.streamables objectForKey:characteristic.UUID];
    NSDictionary *values = [streamable.parser characteristicDataToValues:characteristic.value];
    NSLog(@"%@ %@", characteristic.UUID, values);
    [self.notificationDelegate notification:streamable.identifier values:values];
}

// The rests of these are kept as a place to set breakpoints
#pragma mark CBPeripheralDelegate empty stubs

- (void)                     peripheral:(CBPeripheral *)peripheral
didDiscoverDescriptorsForCharacteristic:(CBCharacteristic *)characteristic
                                  error:(NSError *)error
{
    PERROR(error);
}

- (void)                   peripheral:(CBPeripheral *)peripheral
didDiscoverIncludedServicesForService:(CBService *)service
                                error:(NSError *)error
{
    PERROR(error);
}

- (void)peripheral:(CBPeripheral *)peripheral
 didModifyServices:(NSArray *)invalidatedServices
{
    
}

- (void)         peripheral:(CBPeripheral *)peripheral
didUpdateValueForDescriptor:(CBDescriptor *)descriptor
                      error:(NSError *)error
{
    PERROR(error);
}

- (void)            peripheral:(CBPeripheral *)peripheral
didWriteValueForCharacteristic:(CBCharacteristic *)characteristic
                         error:(NSError *)error
{
    PERROR(error);
}

- (void)        peripheral:(CBPeripheral *)peripheral
didWriteValueForDescriptor:(CBDescriptor *)descriptor
                     error:(NSError *)error
{
    PERROR(error);
}

- (void)peripheralDidUpdateName:(CBPeripheral *)peripheral
{
    
}

- (void)peripheralDidUpdateRSSI:(CBPeripheral *)peripheral
                          error:(NSError *)error
{
    PERROR(error);
}

@end
