/*

 File: LeDiscovery.m
 
 Abstract: Scan for and discover nearby LE peripherals with the 
 matching service UUID.
 
 Version: 1.0
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by 
 Apple Inc. ("Apple") in consideration of your agreement to the
 following terms, and your use, installation, modification or
 redistribution of this Apple software constitutes acceptance of these
 terms.  If you do not agree with these terms, please do not use,
 install, modify or redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software. 
 Neither the name, trademarks, service marks or logos of Apple Inc. 
 may be used to endorse or promote products derived from the Apple
 Software without specific prior written permission from Apple.  Except
 as expressly stated in this notice, no other rights or licenses, express
 or implied, are granted by Apple herein, including but not limited to
 any patent rights that may be infringed by your derivative works or by
 other works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2011 Apple Inc. All Rights Reserved.
 
 */


#import "LeDiscovery.h"
#import "SMHAlertController.h"


@interface LeDiscovery () <CBCentralManagerDelegate, CBPeripheralDelegate> {
	CBCentralManager    *centralManager;
	BOOL				pendingInit;
}
@end


@implementation LeDiscovery

@synthesize foundPeripherals;
//@synthesize connectedServices;
@synthesize discoveryDelegate;
//@synthesize peripheralDelegate;



#pragma mark -
#pragma mark Init
/****************************************************************************/
/*									Init									*/
/****************************************************************************/
+ (id)sharedInstance
{
	static LeDiscovery	*this	= nil;

	if (!this)
		this = [[LeDiscovery alloc] init];

	return this;
}

- (id)init
{
    self = [super init];
    
    if (self) {
		pendingInit = YES;
		centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:dispatch_get_main_queue()];
        foundPeripherals = [[NSMutableArray alloc] init];
	}
    
    return self;
}

- (void)dealloc
{
    // We are a singleton and as such, dealloc shouldn't be called.
    assert(NO);
}

#pragma mark -
#pragma mark Discovery
/****************************************************************************/
/*								Discovery                                   */
/****************************************************************************/
- (void)startScanningForUUIDString:(NSString *)uuidString
{
    NSArray *uuidArray = nil;
    
    if (uuidString) {
        uuidArray = [NSArray arrayWithObject:[CBUUID UUIDWithString:uuidString]];
    }
    
	NSDictionary *options = nil;
    
#if 0
    options = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:NO]
                                          forKey:CBCentralManagerScanOptionAllowDuplicatesKey];
#endif
    
    NSLog(@"start scanning for %@", uuidArray ? [uuidArray description] : @"any peripheral");
	[centralManager scanForPeripheralsWithServices:uuidArray options:options];
}

- (void)stopScanning
{
	[centralManager stopScan];
}

#pragma mark -
#pragma mark CBCentralManagerDelegate

- (void)         centralManager:(CBCentralManager *)central
didRetrieveConnectedPeripherals:(NSArray *)peripherals
{
    NSLog(@"didRetrieveConnectedPeripherals: %@", peripherals);
    
	/* Add to list. */
	for (CBPeripheral *peripheral in peripherals) {
		[central connectPeripheral:peripheral options:nil];
	}
    
	[discoveryDelegate discoveryDidRefresh];
}

- (void)centralManager:(CBCentralManager *)central
 didRetrievePeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"didRetrievePeripheral: %@", peripheral);
	[central connectPeripheral:peripheral options:nil];
	[discoveryDelegate discoveryDidRefresh];
}

- (void)            centralManager:(CBCentralManager *)central
didFailToRetrievePeripheralForUUID:(CFUUIDRef)UUID
                             error:(NSError *)error
{
    NSLog(@"didFailToRetrievePeripheralForUUID");
}

- (void)centralManager:(CBCentralManager *)central
 didDiscoverPeripheral:(CBPeripheral *)peripheral
     advertisementData:(NSDictionary *)advertisementData
                  RSSI:(NSNumber *)RSSI
{
    NSLog(@"didDiscoverPeripheral");
    
    // search explicity for our demo device
    NSString *peripheralName = [advertisementData objectForKey:CBAdvertisementDataLocalNameKey];
    if ([peripheralName length] > 0) {
        if (NSOrderedSame == [peripheralName compare:@"Renesas HC Demo"]) {
            NSLog(@"Found demo device: %@", peripheralName);
            [centralManager stopScan];
            if (![foundPeripherals containsObject:peripheral]) {
                [foundPeripherals addObject:peripheral];
            }
            [discoveryDelegate discoveryDidRefresh];
        }
    }
}

- (void)centralManager:(CBCentralManager *)central
  didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"didConnectPeripheral");
    X1Services *service = [[X1Services alloc] initWithPeripheral:peripheral];
    self.connectedService = service;
    service.notificationDelegate = self.notificationDelegate;
    
    [service discover];
	[discoveryDelegate discoveryDidRefresh];
}

- (void)    centralManager:(CBCentralManager *)central
didFailToConnectPeripheral:(CBPeripheral *)peripheral
                     error:(NSError *)error
{
    NSLog(@"didFailToConnectPeripheral: %@ failed: %@", [peripheral name], [error localizedDescription]);
}

- (void) centralManager:(CBCentralManager *)central
didDisconnectPeripheral:(CBPeripheral *)peripheral
                  error:(NSError *)error
{
    NSLog(@"didDisconnectPeripheral");
    [discoveryDelegate disconnected];
	[discoveryDelegate discoveryDidRefresh];
}

- (void)centralManagerDidUpdateState:(CBCentralManager *)central
{
    const CBCentralManagerState bogusState = 0xffff;
    static CBCentralManagerState previousState = bogusState;
    
    NSLog(@"centralManagerDidUpdateState %@", central);
    
	switch ([centralManager state]) {
		case CBCentralManagerStatePoweredOff:
		{
            NSLog(@"centralManagerDidUpdateStateCBCentralManagerStatePoweredOff");
            [self clearDevices];
            [discoveryDelegate discoveryDidRefresh];
            
			/* Tell user to power ON BT for functionality, but not on first run - the Framework will alert in that instance. */
            if (previousState != bogusState) {
                [discoveryDelegate discoveryStatePoweredOff];
            }
            
            SMHAlertController * alert = [SMHAlertController alertControllerWithTitle:@"Bluetooth"
                                                                              message:@"Please turn Bluetooth on"];
            
            UIAlertAction * okAction = [UIAlertAction actionWithTitle:@"Ok"
                                                                style:UIAlertActionStyleDefault
                                                              handler:nil];
            
            [alert addAction:okAction];
            [alert show];

			break;
		}
            
		case CBCentralManagerStateUnauthorized:
		{
			/* Tell user the app is not allowed. */
            NSLog(@"CBCentralManagerStateUnauthorized");
			break;
		}
            
		case CBCentralManagerStateUnknown:
		{
			/* Bad news, let's wait for another event. */
            NSLog(@"CBCentralManagerStateUnknown");
			break;
		}
            
		case CBCentralManagerStatePoweredOn:
		{
            NSLog(@"CBCentralManagerStatePoweredOn");
			pendingInit = NO;
			[discoveryDelegate discoveryDidRefresh];
			break;
		}
            
		case CBCentralManagerStateResetting:
		{
            NSLog(@"CBCentralManagerStateResetting");
			[self clearDevices];
            [discoveryDelegate discoveryDidRefresh];
			pendingInit = YES;
			break;
		}
            
        case CBCentralManagerStateUnsupported:
        {
            NSLog(@"CBCentralManagerStateUnsupported");
            
            // Mostly for the simulator
            SMHAlertController * alert = [SMHAlertController alertControllerWithTitle:@"Bluetooth Unsupported"
                                                                              message:@"This application requires Bluetooth"];
            
            UIAlertAction * okAction = [UIAlertAction actionWithTitle:@"Ok"
                                                                style:UIAlertActionStyleDefault
                                                              handler:nil];
            
            [alert addAction:okAction];
            [alert show];

            break;
        }
	}
    
    previousState = [centralManager state];
}

#pragma mark -
#pragma mark Connection/Disconnection

- (void)connectPeripheral:(CBPeripheral*)peripheral
{
    if (CBPeripheralStateConnected != peripheral.state) {
		[centralManager connectPeripheral:peripheral options:nil];
	}
}

- (void)disconnectPeripheral:(CBPeripheral*)peripheral
{
	[centralManager cancelPeripheralConnection:peripheral];
    self.connectedService = nil;
}

- (void)disconnect {
    if (self.connectedService) {
        // cancel all characteristic subscriptions
        [self.connectedService setNotifications:NO];
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(250 * NSEC_PER_MSEC)), dispatch_get_main_queue(), ^{
            NSLog(@"disconnect");
            [centralManager cancelPeripheralConnection:self.connectedService.peripheral];
        });
    }
}

- (void)clearDevices
{
    [foundPeripherals removeAllObjects];
}

@end
