/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * pulse_adc.h
 *
 *  Created on: Jan 8, 2016
 *      Author: alejandro
 ****************************************************************************************
 */

#ifndef PULSE_ADC_H_
#define PULSE_ADC_H_

#include "led_pulse.h"
// TODO: wrap this header include depending on MCU used
#include "pulse_adc_ssp.h"

#define R_LED_FREQ_HZ       20000
#define R_LED_DUTY_PERCENT  95 // 01 // 99
#define IR_LED_FREQ_HZ      20000
#define IR_LED_DUTY_PERCENT 95 // 01// 99

#define DAC_BITS 12 // 12 bit DAC out
#define ADC_BITS 14 // 14 bit ADC in

typedef enum
{
    PULSE_ADC_IDLE,
    PULSE_ADC_SAMPLE_START,
    PULSE_ADC_R_SETTLE,
    PULSE_ADC_R_SAMPLE,
    PULSE_ADC_IR_SETTLE,
    PULSE_ADC_IR_SAMPLE,
    PULSE_ADC_MAX_STEPS,
}pulse_adc_sample_step_t;

typedef enum
{
    ADC_TRANS_AMP_AOUT,
    ADC_TRANS_AMP_NEG,
    ADC_TRANS_AMP_POS,
    ADC_GAIN_AMP_OUT,
    ADC_GAIN_AMP_POS,
    ADC_GAIN_AMP_NEG,
    ADC_FEEDBACK_COUNT
}adc_feedbac_t;

enum
{
    PULSE_EVENT_NONE = 0,
    PULSE_R_EVENT_FLAG = (1 << PULSE_LED_RED),
    PULSE_IR_EVENT_FLAG = (1 << PULSE_LED_IRED),
    PULSE_EVENT_LAST = PULSE_IR_EVENT_FLAG
};

typedef struct
{
    int channel;
    int value;
}adc_feedback_t;

typedef adc_feedback_t pulse_adc_sample_led_t;

typedef struct
{
    const pmcu_resource_t *res;  // mcu specific resource context
    pulse_adc_sample_step_t step;
    pulse_adc_sample_led_t sample[PULSE_LED_COUNT];
    adc_feedback_t feedback[ADC_FEEDBACK_COUNT];
}pulse_adc_sample_context_t;

/*
 * MCU specific resources
 */
extern const pmcu_resource_t pulse_adc_mcu_res;

/*
 * LED drive and Feedback ADC resource control context
 */
extern pulse_adc_sample_context_t  pulse_adc_sample_context;

/**
 * @brief pulse_adc_process_red process the adc data for the RED LED
 *
 * @param value ADC value
 *
 * @return none
 */
void pulse_adc_process_ired (int value, bool prev_gTrack, uint16_t *heart_rate, uint16_t *o2_saturation);

/**
 * @brief pulse_adc_process_red process the adc data for the Infre RED LED
 *
 * @param value ADC value
 *
 * @return none
 */
void pulse_adc_process_red (int value, bool prev_gTrack);

/**
 * @brief pulse_adc_init initializes all related to PULSE ADC sensing
 *
 * @param none
 *
 * @return 0 if no error, error code otherwise
 */
int pulse_adc_init (void);

/**
 * @brief pulse_adc_notify_init initializes all related to PULSE ADC data notification
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int pulse_adc_notify_init (const pmcu_resource_t *c);

/**
 * @brief pulse_adc_notify notifies of a particular LED PULSE been sensed
 *
 * @param c pointer to the MCU resource control context
 * @param event contains the flag corresponding to the LED data that is been acquired
 *
 * @return 0 if no error, error code otherwise
 */
int pulse_adc_notify (const pmcu_resource_t *c, int event);

/**
 * @brief pulse_adc_wait_for_event waits for ever for a pulse adc value to be collected event
 *
 * @param event pointer to the event storage
 * @param red pointer to the RED PULSE ADC value (in case RED PULSE event triggered)
 * @param ired pointer to the Infra RED PULSE ADC value (in case Infra RED PULSE event triggered)
 *
 * @return 0 if no error, error code otherwise
 */
int pulse_adc_wait_for_event (int *event, int *red, int *ired);

/**
 * @brief sample_adc_open opens ADC peripheral, leaves it ready
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int sample_adc_open (const pmcu_resource_t *c);

/**
 * @brief sample_timer_open opens 10ms timer and starts it
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int sample_timer_open (const pmcu_resource_t *c);

/**
 * @brief sample_settle_timer_open opens 4ms single shot timer (does not start)
 *
 * @param c pointer to the MCU resource control context
 *
 * @return 0 if no error, error code otherwise
 */
int sample_settle_timer_open (const pmcu_resource_t *c);

/**
 * @brief adc_settle_timer_start fires up the settle timer (4ms)
 *
 * @param c pointer to the MCU resource control context
 *
 * @return none
 */
void adc_settle_timer_start (const pmcu_resource_t *c);

/**
 * @brief adc_convert_start fires up the ADC conversion
 *
 * @param c pointer to the MCU resource control context
 *
 * @return none
 */
void adc_convert_start (float v, const pmcu_resource_t *c);

/**
 * @brief pulse_adc_sample_read reads the converted value from the PULSE_ADC line
 *
 * @param c pointer to the pulse sample context
 * @param c pointer to the adc sample return storage area
 *
 * @return 0 if no error, error code otherwise
 */
int pulse_adc_sample_read (pulse_adc_sample_context_t *c, pulse_adc_sample_led_t *sample);

/**
 * @brief pulse_adc_sample_start_start initiates the process of sampling the LEDs
 *
 * @param c pointer to the pulse sample context
 *
 * @return none
 */
void pulse_adc_sample_start_start (pulse_adc_sample_context_t *c);

/**
 * @brief pulse_adc_sample_arm_handle handles the multiple states of LEDs sampling
 *
 * @param c pointer to the pulse sample context
 *
 * @return none
 */
void pulse_adc_sample_arm_handle (pulse_adc_sample_context_t *c);

/**
 * @brief pulse_adc_sample_handle samples the ADC channel associated with the PULSE ADC
 *
 * @param c pointer to the pulse sample context
 *
 * @return none
 */
void pulse_adc_sample_handle (pulse_adc_sample_context_t *c);

/**
 * @brief pulse_adc_percent returns the percend that the ADC is
 *
 * @param adc ADC raw count value
 *
 * @return percent 0 - 100
 */
float pulse_adc_percent (int adc);

/****************
 * below are the HAL layer routines provided by user
 ****************/

/**
 * @brief hal_sample_adc_open hardware specific refer to sample_adc_open
 */
int hal_sample_adc_open (const pmcu_resource_t *c);
/**
 * @brief hal_sample_timer_open hardware specific refer to sample_timer_open
 */
int hal_sample_timer_open (const pmcu_resource_t *c);
/**
 * @brief hal_sample_settle_timer_open hardware specific refer to sample_settle_timer_open
 */
int hal_sample_settle_timer_open (const pmcu_resource_t *c);
/**
 * @brief hal_adc_settle_timer_start hardware specific refer to adc_settle_timer_start
 */
void hal_adc_settle_timer_start (const pmcu_resource_t *c);
/**
 * @brief hal_adc_idle_timer_start hardware specific refer to adc_idle_timer_start
 */
void hal_adc_idle_timer_start (const pmcu_resource_t *c);
/**
 * @brief hal_adc_convert_start hardware specific refer to adc_convert_start
 */
void hal_adc_convert_start (const pmcu_resource_t *c);
/**
 * @brief hal_pulse_adc_sample_read hardware specific refer to pulse_adc_sample_read
 */
int hal_pulse_adc_sample_read (pulse_adc_sample_context_t *c, pulse_adc_sample_led_t *sample);
/**
 * @brief hal_pulse_adc_notify_init hardware specific refer to pulse_adc_notify_init
 */
int hal_pulse_adc_notify_init (const pmcu_resource_t *c);
/**
 * @brief hal_pulse_adc_notify hardware specific refer to pulse_adc_notify
 */
int hal_pulse_adc_notify (const pmcu_resource_t *c, int event);
/**
 * @brief hal_pulse_adc_wait_for_event hardware specific refer to pulse_adc_notify_init
 */
int hal_pulse_adc_wait_for_event (pulse_adc_sample_context_t *c, int *event, int *value);

#endif /* PULSE_ADC_H_ */
