/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "pulse_ox_thread.h"
#include "util.h"
#include "display.h"
#include "hal_data.h"
#include "led_pulse.h"
#include "pulse_adc.h"
#include "main_thread.h"
#include "pulse_adc_ssp.h"
#include "led.h"
#include "events.h"
#include "pulse_adc_process.h"
#include "bsp_api.h"
#include "common.h"
#include "application_dispatcher_thread.h"

//#define DISPLAY_VOLTAGE_PERCENT
#define TIMEOUT_RELATED_TOGGLE

#define TOGGLE_TIMEOUT_SEC  5

// some demo limits
#define HR_MIN_LIMIT        50
#define HR_MAX_LIMIT        200
#define O2_MIN_LIMIT        96
#define O2_MAX_LIMIT        100

extern healthcare_data g_healthcare_data;

void pulse_ox_thread_entry(void);
void pulse_bias_init (void);

static float adc2percent (int adc);
static int pulse_ox_init (void);
static int pulse_ox_init (void);
static void handle_display_info (void);

static uint16_t heart_rate;    // These are the output results
static uint16_t o2_saturation; // These are the output results

static pulse_ox_context_t context = {
    .event = &pulse_ox_app_event,
    .mutex = &pulse_adc_value_mutex,
    .idle = true,
};

/*******************************************************************************************

 Pulse and Oximetry measuring thread entry function

*******************************************************************************************/

void pulse_ox_thread_entry(void)
{
    static bool prev_gTrack = false;

    int err = 0, red, ired, event;
    pulse_ox_context_t *c = &context;

    err = pulse_ox_init ();
    Set_Offset_v2 ( OFFSET_INIT );

    // start out in idle state
    pulse_ox_idle_enter();

    while (0 == err)
    {
        // only run if in idle state
        if (!c->idle)
        {
            // wait for RED led OR Infra RED led ADC value to be available
            err = pulse_adc_wait_for_event (&event, &red, &ired);
            // block
            if (TX_SUCCESS == tx_mutex_get (c->mutex, TX_NO_WAIT))
            {
                c->red = red;
                c->ired = ired;
                // release
                tx_mutex_put (c->mutex);
            }
            else
            {
                // we could not get the lock ... we lost this sample ... just track
                c->stat_lost++;
            }

            if (event & PULSE_IR_EVENT_FLAG)
            {
                pulse_adc_process_ired (c->red, prev_gTrack, &heart_rate, &o2_saturation);
            }

            if (event & PULSE_R_EVENT_FLAG)
            {
                pulse_adc_process_red (c->ired, prev_gTrack);
                prev_gTrack = true;
            }

            handle_display_info ();
        }
        else
        {
            led_all_off ();                                                                           //Turn off all LEDs
            // blank out LCD
            display_str("   ");
            // clear out BLE reported data
            tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
            memset(&g_healthcare_data, 0, sizeof(g_healthcare_data));
            tx_mutex_put(&g_data_mutex);

            tx_thread_sleep (10);
        }

    }

    while (1)
    {
        // get stuck here
        tx_thread_sleep (1);
    }
}

/*******************************************************************************************/
void pulse_ox_get (int *red, int *ired)
{
    //
    pulse_ox_context_t *c = &context;

    tx_mutex_get (c->mutex, TX_WAIT_FOREVER);
    //
    if (red)
    {
        *red = c->red;
    }

    if (ired)
    {
        *ired = c->ired;
    }
    //
    tx_mutex_put (c->mutex);
}

/*******************************************************************************************/
void pulse_ox_get_hr (uint16_t *hr)
{
    //
    pulse_ox_context_t *c = &context;

    tx_mutex_get (c->mutex, TX_WAIT_FOREVER);
    //
    if (hr)
    {
        *hr = c->results.hr;
    }
    //
    tx_mutex_put (c->mutex);
}

/*******************************************************************************************/
void pulse_ox_get_o2 (uint16_t *o2)
{
    //
    pulse_ox_context_t *c = &context;

    tx_mutex_get (c->mutex, TX_WAIT_FOREVER);
    //
    if (o2)
    {
        *o2 = c->results.o2;
    }
    //
    tx_mutex_put (c->mutex);
}

/*******************************************************************************************/
static void  power_button_hanler (bsp_grp_irq_t irq)
{
    pulse_ox_context_t *c = &context;
    // toggle
    c->idle = !c->idle;
    // entering?
    if (c->idle)
    {
        // inform on IDLE entering state
        tx_event_flags_set (c->event, IDLE_ENTER_EVENT, TX_OR);
    }
    else
    {
        // inform on IDLE leaving state
        tx_event_flags_set (c->event, IDLE_EXIT_EVENT, TX_OR);
    }
}

/*******************************************************************************************/
static void NMI_pin_init (void)
{
    R_ICU->NMICR_b.NMIMD = 0;   // falling edge
    R_ICU->NMICR_b.NFCLKSEL = 0x03; // PCLKB/64
    R_ICU->NMICR_b.NFLTEN = 1; // Digital filter is enabled
    //
    R_BSP_GroupIrqWrite(BSP_GRP_IRQ_NMI_PIN, power_button_hanler);

    R_ICU->NMIER_b.NMIEN = 1; // NMI pin interrupt is enabled
}

/*******************************************************************************************/
static int pulse_ox_init (void)
{
    // sampling of the SAMPLE_ADC line
    int err = pulse_adc_init ();

    if (0 == err)
    {
        // DAC for PULSE_BIAS_DAC
        pulse_bias_init ();
    }

    if (0 == err)
    {
        display_open ();
    }

    if (0 == err)
    {
        NMI_pin_init ();
    }

    return err;
}

/*******************************************************************************************/
static void handle_display_info (void)
{
    static int it = 0;
    pulse_ox_context_t *c = &context;

/*******************************************************************************************/
#ifdef DISPLAY_VOLTAGE_PERCENT

    // track it
    if (0 == (it++ % 100))
    {
        // temporary
        float p = pulse_adc_percent (c->red);
        //
        display (p);
    }

/*******************************************************************************************/
#elif defined (TIMEOUT_RELATED_TOGGLE)
    if (0 == (++it % (TOGGLE_TIMEOUT_SEC * LED_SAMPLE_RATE * PULSE_LED_COUNT)))
    {
        // pretend this to be a button up event
        tx_event_flags_set (c->event, BTN_UP_EVENT, TX_OR);
    }
#else
    // NOTE: ... do nothing .. this will be handled differently
#endif
}

/*******************************************************************************************/
void pulse_ox_update (uint16_t hr, uint16_t o2)
{
    pulse_ox_context_t *c = &context;
    // block
    tx_mutex_get (c->mutex, TX_WAIT_FOREVER);
    tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);

    // only update HR if values are within demo range
    if ((hr <= HR_MAX_LIMIT && hr >= HR_MIN_LIMIT))
    {
        c->results.hr = hr;
        g_healthcare_data.heart_rate = hr;

        // only update O2 if values are within demo range
        if (o2 > O2_MAX_LIMIT)
        {
            o2 = O2_MAX_LIMIT;
        }
        if (02 < O2_MIN_LIMIT)
        {
            o2 = O2_MIN_LIMIT;
        }

        c->results.o2 = o2;
        g_healthcare_data.spo2 = o2;

        led_green(STATUS_LED);
    }
    else
    {
        c->results.hr = 0;
        g_healthcare_data.heart_rate = 0;
        c->results.o2 = 0;
        g_healthcare_data.spo2 = 0;
        led_red(STATUS_LED);
    }

    // release mutext lock
    tx_mutex_put (c->mutex);
    tx_mutex_put(&g_data_mutex);
}
