/**
 ****************************************************************************************
 *
 * @file plf.c
 *
 * @brief This file contains the Low Level RENESAS RL78 MCU clock initialisation functions.
 *
 * Copyright (C) RivieraWaves 2009-2012
 *
 * Copyright(C) 2012-2014 Renesas Electronics Corporation
 *
 * $Rev: 19805 $
 *
 * rev.214
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup PLATFORM_DRIVERS
 * @{
 ****************************************************************************************
 */
#ifdef _USE_IAR_RL78
#define _CLK_TBL _Pragma("location=\"CLK_TBL\"")
#define _PLF_CODE _Pragma("location=\"PLF_CODE\"")
#else // _USE_IAR_RL78
#define _CLK_TBL
#define _PLF_CODE

#if !defined(_USE_CCRL_RL78)
#pragma section @@CODEL PLF_CODE
#pragma section @@CNST  CLK_TBL
#else
#pragma section text    PLF_CODE
#pragma section const   CLK_TBL
#endif
#endif // _USE_IAR_RL78


/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include "arch.h"

///Include corresponding header file
#include "ll.h"

#include "port.h"
#include "plf.h"
#include "hw_config.h"

/* CMC register settings(1/2) */
#if defined(CLK_HOCO)
/* High-speed on-chip oscillator */
#define SFR_VAL_CMC_MSYSCLK (0x00U)
#elif defined(CLK_X1_LOW)
/* X1 oscillator (fX <= 10MHz) */
#define SFR_VAL_CMC_MSYSCLK (0x40U)
#elif defined(CLK_X1_HIGH)
/* X1 oscillator (fX > 10MHz) */
#define SFR_VAL_CMC_MSYSCLK (0x41U)
#elif defined(CLK_EX)
/* External main system clock */
#define SFR_VAL_CMC_MSYSCLK (0xC0U)
#else
#error "an unsupported main system clock configuration."
#endif

/* CMC register settings(2/2) */
#if defined(CLKS_XT1)
/* XT1 oscillator */
#define SFR_VAL_CMC_SSYSCLK (0x10U)
#elif defined(CLKS_EX)
/* External clock input */
#define SFR_VAL_CMC_SSYSCLK (0x30U)
#else
#define SFR_VAL_CMC_SSYSCLK (0x00U)
#endif

/* OSTS register setting */
#define SFR_VAL_OSTS        (0x02U)
/* Format of Oscillation Stabilization Time Counter Status */
/* compare OSTC register and this value                    */
#define VAL_X1_OSCSTAB_MSK  (0xE0U)

#if defined(CLK_HOCO)
#define SFRB_VAL_MSTOP      1
#define SFRB_VAL_MCM0       0
#define SFRB_VAL_HIOSTOP    0
#else
#define SFRB_VAL_MSTOP      0
#define SFRB_VAL_MCM0       1
#define SFRB_VAL_HIOSTOP    1
#endif

#if defined(CLKS_XT1) || \
    defined(CLKS_EX)
#define SFRB_VAL_XTSTOP     0
#else
#define SFRB_VAL_XTSTOP     1
#endif

#if defined(CLKS_XT1) || \
    defined(CLKS_EX)
/* Enables supply of subsystem clock to peripheral functions */
#define SFR_VAL_OSMC        (0x00U)
#else
/* Selection of count clock for real-time clock and 12-bit   */
/* interval timer: Low-speed on-chip oscillator clock (fIL)  */
#define SFR_VAL_OSMC        (0x10U)
#endif

#define SFRB_VAL_MCS_fMX    1

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */
#ifndef CFG_USE_APLILET
#ifdef _USE_IAR_RL78
///Option Byte section definition
__root __far const uint8_t ll_option_bytes[OPT_BYTES_SIZE]  @ 0x00C0=
                                                                       {WATCHDOG_DISABLED,
                                                                        LVD_MODE_1_63V,
                                                                        FRQ_8MHZ,
                                                                        OCD_ENABLED,};

///Security Byte Definition
__root __far const uint8_t ll_secu_id[SECU_ID_SIZE] @ 0x00C4=
                                                         { 0xff, 0xff, 0xff, 0xff, 0xff,
                                                           0xff, 0xff, 0xff, 0xff, 0xff };
#endif //_USE_STD_C99
#endif //CFG_USE_APLILET

#define CLK_OPTIONBYTE  ((__far uint8_t *)(0x0000C0))

/*
 * FUNCTION DEFINITIONS
 ****************************************************************************************
 */
#ifdef USE_FW_UPDATE_PROFILE
extern bool check_fw_update( void );
#endif
#ifdef CFG_USE_APLILET
extern void R_Systeminit(void);
#endif

/**
 *****************************************************************************************
 * @brief Static function for setting for the BLE HW interrupt
 *****************************************************************************************
 */
_PLF_CODE static void plf_irq_init(void)
{
    //falling edge detection
    write1_sfr(EGN1, 3, 1);

    //set interrupt priority as level2
    write1_sfrbit(PPR111, 1);
    write1_sfrbit(PPR011, 0);

    //clear interrupt request flag
    write1_sfrbit(PIF11, IRQ_CLEAR);

    //enable ext. INTP11 interrupt
    write1_sfrbit(PMK11, IRQ_ENABLE);
}

/**
 *****************************************************************************************
 * @brief Static function for the RF to setting Clock Output as sleep clock
 *****************************************************************************************
 */
_PLF_CODE static void plf_clkoutput_init(const uint8_t plf_flg)
{
    //Configuration of Clock Output
    switch(plf_flg)
    {
        case PLF_PCLBUZ_16KHZ:
            write_sfr(CKS0, 0x09); /* Supply of 16.384kHz clock */
            break;
        case PLF_PCLBUZ_32KHZ:
            write_sfr(CKS0, 0x08); /* Supply of 32.768kHz clock */
            break;
        default:
            /* no clock output */
            return;
    }

#ifdef CLKS_XT1
    write1_sfr(P7,  6, 1);
    write1_sfr(PM7, 6, 0);
#endif

    //Output port settings
    write1_sfr(PM14, 0, 0);
    write1_sfr(P14, 0, 0);

    write1_sfrbit(PCLOE0, 1); /* Output enable */
}

/**
 *****************************************************************************************
 * @brief Static function for port setting
 *****************************************************************************************
 */
_PLF_CODE static void plf_port_init(void)
{
#ifndef CONFIG_EMBEDDED
    /* set the serial output pin, to avoid falling low-level */
    write1_sfr(P1, 2, PORT_LATCH);  // for UART-TxD0
    /* set the serial direction pin, to avoid falling low-level */
    write1_sfr(P2, 1, PORT_LATCH);  // for SDIR
#endif

    //P0.2 Digital i/o
    write1_sfr(PMC0, 2, 0);
    //P0.3 Digital i/o
    write1_sfr(PMC0, 3, 0);
    //A/D port configuration
    write_sfr(ADPC, 0x01);

#ifndef CFG_USE_APLILET
    /*Pin I/O mode setting*/
    write_sfr(PM0,  0xEF);
    write_sfr(PM1,  0x7F);
    write_sfr(PM2,  0xFF);
    write_sfr(PM3,  0xFD);
    write_sfr(PM4,  0xF1);
    write_sfr(PM5,  0xC0);
    write_sfr(PM6,  0xF3);
    write_sfr(PM7,  0xF7);
    write_sfr(PM12, 0xFF);
    write_sfr(PM14, 0xBD);
#endif
}

_PLF_CODE void plf_init(const uint8_t plf_flg)
{
    uint16_t    loop_cnt;
    uint8_t     op_byte = (uint8_t)(CLK_OPTIONBYTE[2]);
#if defined(CLK_X1_LOW) || \
    defined(CLK_X1_HIGH)
    uint8_t     temp_stab_set, temp_stab_wait;
#endif

    // chg mcu clock
    write_sfr(HOCODIV, (uint8_t)(op_byte & 0x07));      // equal optionbyte

    // calc waitloop
    clk_waitinit();

    //port settings for port mode(A/D)
    plf_port_init();

#ifdef CFG_USE_APLILET
    R_Systeminit();
#else //CFG_USE_APLILET
    /* CMC(Clock operation mode control) register */
    write_sfr(CMC, (SFR_VAL_CMC_MSYSCLK | SFR_VAL_CMC_SSYSCLK));

#if defined(CLK_X1_LOW) || \
    defined(CLK_X1_HIGH)
    /* OSTS(Oscillation stabilization time select) register */
    write_sfr(OSTS, SFR_VAL_OSTS);
#endif

    /* MSTOP bit                                      */
    /* 0: X1 oscillator or External main system clock */
    /* 1: High-speed on-chip oscillator               */
    write1_sfrbit(MSTOP, SFRB_VAL_MSTOP);

#if defined(CLK_X1_LOW) || \
    defined(CLK_X1_HIGH)
    /* X1 Oscillation stabilization wait */
    temp_stab_set = VAL_X1_OSCSTAB_MSK;
    do
    {
        temp_stab_wait  = read_sfr(OSTC);
        temp_stab_wait &= temp_stab_set;
    }
    while (temp_stab_wait != temp_stab_set);
#endif

    /* MCM0 bit (Main system clock (fMAIN) operation control)                                    */
    /* 0: Selects the high-speed on-chip oscillator clock (fIH) as the main system clock (fMAIN) */
    /* 1: Selects the high-speed system clock (fMX) as the main system clock (fMAIN)             */
    write1_sfrbit(MCM0, SFRB_VAL_MCM0);

    /* XTSTOP bit (Subsystem clock operation control)                         */
    /* 0: XT1 oscillator operating or External clock from EXCLKS pin is valid */
    /* 1: XT1 oscillator stopped or External clock from EXCLKS pin is invalid */
    write1_sfrbit(XTSTOP, SFRB_VAL_XTSTOP);

#if defined(CLKS_XT1)
    // XT1 oscillation stabilization wait time.
    /* Change the waiting time according to the system */
    for( loop_cnt=0; loop_cnt<1000; loop_cnt++ ){
        // about 1s.
        clk_waitfunc( CLK_WAIT_1000_USEC );         // 1msec x 1000 loop
    }
#endif

    /* OSMC(Subsystem clock supply mode control) register */
    write_sfr(OSMC, SFR_VAL_OSMC);

    /* CSS bit (Selection of CPU/peripheral hardware clock) */
    /* always Main system clock (fMAIN)                     */
    write1_sfrbit(CSS, 0);

#ifdef USE_FW_UPDATE_PROFILE
    if( true == check_fw_update() )
    {
        // When the FW update, use the High-speed on-chip oscillator clock
        write1_sfrbit(HIOSTOP, 0);
    }
    else
    {
#endif
        /* HIOSTOP bit                                */
        /* Set fIH                                    */
        /* 0: High-speed on-chip oscillator operating */
        /* 1: High-speed on-chip oscillator stopped   */
        write1_sfrbit(HIOSTOP, SFRB_VAL_HIOSTOP);
#ifdef USE_FW_UPDATE_PROFILE
    }
#endif
#endif //CFG_USE_APLILET

    //port settings for interrupt pin from BLE core
    plf_irq_init();

    if(plf_flg != PLF_PCLBUZ_NONE)
    {
        //Configuration of Clock Output
        plf_clkoutput_init(plf_flg);
    }

    return;
}

_PLF_CODE void plf_hoco_start(void)
{
    /* if Status of Main system clock (fMAIN) is High-speed system clock (fMX) */
    if(read1_sfr(CKC, 5) == SFRB_VAL_MCS_fMX)
    {
        /* High-speed on-chip oscillator operating */
        write1_sfrbit(HIOSTOP, 0);
        /* stabilization wait time */
        clk_waitfunc(CLK_WAIT_30_USEC);
    }
}

_PLF_CODE void plf_hoco_stop(void)
{
    /* if Status of Main system clock (fMAIN) is High-speed system clock (fMX) */
    if(read1_sfr(CKC, 5) == SFRB_VAL_MCS_fMX)
    {
        /* High-speed on-chip oscillator stopped   */
        write1_sfrbit(HIOSTOP, 1);
    }
}


_CLK_TBL const uint16_t clk_table_ent[6] = {
#ifdef _USE_REL_RL78
#if defined(CLK_FCLK_4MHZ)
    #if CFG_CON == 1
        10, 40,162, 94,108, 5
    #elif CFG_CON == 2
        10, 40,194, 94,128, 5
    #elif CFG_CON == 3
        10, 40,240, 94,158, 5
    #elif CFG_CON == 4
        10, 40,286, 94,188, 5
    #elif CFG_CON == 5
        10, 40,332, 94,216, 5
    #elif CFG_CON == 6
        10, 40,380, 94,246, 5
    #elif CFG_CON == 7
        10, 40,426, 94,176, 5
    #elif CFG_CON == 8
        10, 40,472, 94,304, 5
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_8MHZ)
    #if CFG_CON == 1
         7, 80, 81, 28, 54, 3
    #elif CFG_CON == 2
         7, 80, 97, 28, 64, 3
    #elif CFG_CON == 3
         7, 80,120, 28, 79, 3
    #elif CFG_CON == 4
         7, 80,143, 28, 94, 3
    #elif CFG_CON == 5
         7, 80,166, 28,108, 3
    #elif CFG_CON == 6
         7, 80,190, 28,123, 3
    #elif CFG_CON == 7
         7, 80,213, 28,138, 3
    #elif CFG_CON == 8
         7, 80,236, 28,152, 3
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_16MHZ)
    #if CFG_CON == 1
         6,160, 68, 26, 43, 3
    #elif CFG_CON == 2
         6,160, 82, 26, 51, 3
    #elif CFG_CON == 3
         6,160,102, 26, 62, 3
    #elif CFG_CON == 4
         6,160,123, 26, 74, 3
    #elif CFG_CON == 5
         6,160,143, 26, 86, 3
    #elif CFG_CON == 6
         6,160,163, 26, 97, 3
    #elif CFG_CON == 7
         6,160,117, 26,109, 3
    #elif CFG_CON == 8
         6,160,204, 26,121, 3
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_32MHZ)
    #if CFG_CON == 1
         6,320, 62, 22, 37, 3
    #elif CFG_CON == 2
         6,320, 75, 22, 44, 3
    #elif CFG_CON == 3
         6,320, 94, 22, 54, 3
    #elif CFG_CON == 4
         6,320,113, 22, 64, 3
    #elif CFG_CON == 5
         6,320,132, 22, 75, 3
    #elif CFG_CON == 6
         6,320,151, 22, 85, 3
    #elif CFG_CON == 7
         6,320,170, 22, 95, 3
    #elif CFG_CON == 8
         6,320,189, 22,106, 3
    #else
        #error "check CFG_CON"
    #endif
#endif
#endif /* _USE_REL_RL78 */

#ifdef _USE_CCRL_RL78
#if defined(CLK_FCLK_4MHZ)
    #if CFG_CON == 1
        10, 40,162, 50,108, 5
    #elif CFG_CON == 2
        10, 40,194, 50,128, 5
    #elif CFG_CON == 3
        10, 40,240, 50,158, 5
    #elif CFG_CON == 4
        10, 40,286, 50,188, 5
    #elif CFG_CON == 5
        10, 40,332, 50,216, 5
    #elif CFG_CON == 6
        10, 40,380, 50,246, 5
    #elif CFG_CON == 7
        10, 40,426, 50,176, 5
    #elif CFG_CON == 8
        10, 40,472, 50,304, 5
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_8MHZ)
    #if CFG_CON == 1
         7, 80, 81, 15, 54, 3
    #elif CFG_CON == 2
         7, 80, 97, 15, 64, 3
    #elif CFG_CON == 3
         7, 80,120, 15, 79, 3
    #elif CFG_CON == 4
         7, 80,143, 15, 94, 3
    #elif CFG_CON == 5
         7, 80,166, 15,108, 3
    #elif CFG_CON == 6
         7, 80,190, 15,123, 3
    #elif CFG_CON == 7
         7, 80,213, 15,138, 3
    #elif CFG_CON == 8
         7, 80,236, 15,152, 3
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_16MHZ)
    #if CFG_CON == 1
         6,160, 68, 11, 43, 3
    #elif CFG_CON == 2
         6,160, 82, 11, 51, 3
    #elif CFG_CON == 3
         6,160,102, 11, 62, 3
    #elif CFG_CON == 4
         6,160,123, 11, 74, 3
    #elif CFG_CON == 5
         6,160,143, 11, 86, 3
    #elif CFG_CON == 6
         6,160,163, 11, 97, 3
    #elif CFG_CON == 7
         6,160,117, 11,109, 3
    #elif CFG_CON == 8
         6,160,204, 11,121, 3
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_32MHZ)
    #if CFG_CON == 1
         6,320, 62, 9, 37, 3
    #elif CFG_CON == 2
         6,320, 75, 9, 44, 3
    #elif CFG_CON == 3
         6,320, 94, 9, 54, 3
    #elif CFG_CON == 4
         6,320,113, 9, 64, 3
    #elif CFG_CON == 5
         6,320,132, 9, 75, 3
    #elif CFG_CON == 6
         6,320,151, 9, 85, 3
    #elif CFG_CON == 7
         6,320,170, 9, 95, 3
    #elif CFG_CON == 8
         6,320,189, 9,106, 3
    #else
        #error "check CFG_CON"
    #endif
#endif
#endif /* _USE_CCRL_RL78 */

#ifdef _USE_IAR_RL78
#if defined(CLK_FCLK_4MHZ)
    #if CFG_CON == 1
        10, 40,162, 104,108, 5
    #elif CFG_CON == 2
        10, 40,194, 104,128, 5
    #elif CFG_CON == 3
        10, 40,240, 104,158, 5
    #elif CFG_CON == 4
        10, 40,286, 104,188, 5
    #elif CFG_CON == 5
        10, 40,332, 104,216, 5
    #elif CFG_CON == 6
        10, 40,380, 104,246, 5
    #elif CFG_CON == 7
        10, 40,426, 104,176, 5
    #elif CFG_CON == 8
        10, 40,472, 104,304, 5
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_8MHZ)
    #if CFG_CON == 1
         7, 80, 86, 33, 57, 3
    #elif CFG_CON == 2
         7, 80,102, 33, 67, 3
    #elif CFG_CON == 3
         7, 80,125, 33, 82, 3
    #elif CFG_CON == 4
         7, 80,149, 33, 97, 3
    #elif CFG_CON == 5
         7, 80,172, 33,111, 3
    #elif CFG_CON == 6
         7, 80,195, 33,126, 3
    #elif CFG_CON == 7
         7, 80,218, 33,141, 3
    #elif CFG_CON == 8
         7, 80,242, 33,155, 3
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_16MHZ)
    #if CFG_CON == 1
         6,160, 72, 27, 44, 3
    #elif CFG_CON == 2
         6,160, 86, 27, 52, 3
    #elif CFG_CON == 3
         6,160,106, 27, 64, 3
    #elif CFG_CON == 4
         6,160,126, 27, 75, 3
    #elif CFG_CON == 5
         6,160,146, 27, 87, 3
    #elif CFG_CON == 6
         6,160,167, 27, 99, 3
    #elif CFG_CON == 7
         6,160,187, 27,110, 3
    #elif CFG_CON == 8
         6,160,207, 27,122, 3
    #else
        #error "check CFG_CON"
    #endif
#endif

#if defined(CLK_FCLK_32MHZ)
    #if CFG_CON == 1
         6,320, 64, 22, 38, 3
    #elif CFG_CON == 2
         6,320, 77, 22, 45, 3
    #elif CFG_CON == 3
         6,320, 96, 22, 55, 3
    #elif CFG_CON == 4
         6,320,115, 22, 65, 3
    #elif CFG_CON == 5
         6,320,134, 22, 75, 3
    #elif CFG_CON == 6
         6,320,153, 22, 85, 3
    #elif CFG_CON == 7
         6,320,172, 22, 95, 3
    #elif CFG_CON == 8
         6,320,191, 22,106, 3
    #else
        #error "check CFG_CON"
    #endif
#endif
#endif /* _USE_IAR_RL78 */
};


/// @} module
